Ti.include('redux.js');

// Important: To make split view work...
// Setup and open split view but keep hidden.
// Open home window over the top
// Any buttons clicked on homeWindow sets splitView visible to true and homeWindow visible to false

// Include our utilities in every JS context
includeGlobal('includes/underscore.js', 
	'includes/utilities.js', 
	'includes/properties.js', 
	'includes/widget_factory.js',
	'includes/remote_api.js');

// Include our RJSS
//includeRJSSGlobal('rjss/common.rjss');

// Control variables for window loadding/unloading
Ti.App.loadedWindows = [];

clearSessionSelections();

//SIDEBAR
var listWin = Ti.UI.createWindow({
    //url:'screens/ipad/blank.js',
    backgroundImage:imagePath('bg-iPad-leftnav.png')
});
listWin.hideNavBar();
 
//DETAIL
var detailWin = new Window({
	//url:'screens/ipad/blank.js'
});
 
 
//SPLIT WINDOW
var listNavGroup = Ti.UI.iPhone.createNavigationGroup({
    window: listWin
});
 
var detailNavGroup = Ti.UI.iPhone.createNavigationGroup({
    window: detailWin
});
 
var splitWin = Ti.UI.iPad.createSplitWindow({
    masterView: listNavGroup,
    detailView: detailNavGroup,
    orientationModes:[
    	Titanium.UI.LANDSCAPE_LEFT,
    	Titanium.UI.LANDSCAPE_RIGHT,
    ],
});
 
	
function openSplitView(show) {
	splitWin.open();
	splitWin.visible = show;
}

// Open this before any views/tableviews so it becomes the root controller.	 
openSplitView(false);


// HOME WINDOW
var homeWindow = new Window({
	title:'Home', 
	backgroundImage:'images/background-iPad.png',
    orientationModes:[
    	Titanium.UI.LANDSCAPE_LEFT,
    	Titanium.UI.LANDSCAPE_RIGHT,
    ],
});

var bigImage = new View({backgroundImage:'images/menu-full-iPad.png',height:626, width:971, top:80});
homeWindow.add(bigImage);

var homeSearchView = new View({
	backgroundColor:'transparent',
	left:40,
	right:40,
	top:30,
	height:30
});
var searchLabel = new Label({
	text:'Search events by:', 
	color: '#FFF', 
	backgroundColor:'transparent',
	font: { fontSize: 18, fontWeight:'bold' }, 
	left:0, 
	top:0 
});
homeSearchView.add(searchLabel);
var magnifyingGlass = new ImageView({backgroundImage:'images/ico_search.png', width:18, height:18});
var searchField = new TextField({
    color:'#000',
    height:30,
    width:780,
    top:0,
    right:0,
    borderStyle:Titanium.UI.INPUT_BORDERSTYLE_ROUNDED,
    borderRadius:17,
    clearButtonMode:Ti.UI.INPUT_BUTTONMODE_ONFOCUS,
    keyboardToolbarColor: '#999',   
    keyboardToolbarHeight: 40,
    leftButton:magnifyingGlass
});
homeSearchView.add(searchField);

magnifyingGlass.addEventListener('click', function(event) { searchField.focus(); });

searchField.addEventListener('return', function(event) {
	loadSearchResultsWindow(event.source.value);
});

homeWindow.add(homeSearchView);

// Date virtual button
var dateButton = new View({borderWidth:1, borderColor:'transparent', height:225, width:225, left:70, top:150});
homeWindow.add(dateButton);
$(dateButton).click(function(event){
	loadViewByDateBaseWindow();
});

// Category virtual button
var categoryButton = new View({borderWidth:1, borderColor:'transparent', height:290, width:290, left:320, top:250});
homeWindow.add(categoryButton);
$(categoryButton).click(function(event){
	loadViewByCategoryBaseWindow();
});

// Location virtual button
var locationButton = new View({borderWidth:1, borderColor:'transparent', height:255, width:255, left:690, top:130});
homeWindow.add(locationButton);
$(locationButton).click(function(event){
	loadViewByLocationBaseWindow();
});

// Random virtual button
var randomButton = new View({borderWidth:1, borderColor:'transparent', height:160, width:160, left:615, top:387});
homeWindow.add(randomButton);
$(randomButton).click(function(event){
	loadRandomBaseWindow();
});

// About virtual button
var aboutButton = new View({borderWidth:1, borderColor:'transparent', height:55, width:55, left:57, top:640});
homeWindow.add(aboutButton);
$(aboutButton).click(function(event){
	loadInfoBaseWindow();
});

var suggestButton = new Button({
	backgroundImage: imagePath('turquoise/btn.png'),
	title: 'suggest an event',
	font: { fontSize: 15, fontWeight: 'bold', },
	width: 170,
	height: 44,
	right:20,
	bottom: 30,
})
homeWindow.add(suggestButton);
$(suggestButton).click(function(event){
	loadSuggestBaseWindow();
});

homeWindow.open();
checkInternet();
// END - HOME WINDOW


// MASTER VIEW
function makeRow(rowId, rowTitle, icon, icon_height, icon_width, callback) {
	var r =  new TableViewRow({
		id:rowId, 
		backgroundColor:'transparent',  
		selectedBackgroundColor:'transparent',
	});
	
	var l = new Label({text:rowTitle, left:40, height:'auto', color:'white', backgroundColor:'transparent', font: { fontSize: 16, fontWeight:'bold' }});
	r.add(l);
	
	if(icon != null) {
		var i = new ImageView({ backgroundImage:icon, width:icon_width, height:icon_height, left:40 });
		r.add(i);
		l.left = 95;
	}

	
	$(r).click(function(event){
		callback();
	})
	
	return r;
}

var logoView = new ImageView({top:40, left:40, height:92, width:215, backgroundImage:'images/logo_vicstate.png'});
listWin.add(logoView);
$(logoView).click(function(e){
	splitWin.visible = false;
	homeWindow.visible = true;
});


var magnifyingGlass = new ImageView({backgroundImage:'images/ico_search.png', width:18, height:18});
var miniSearchBox = new TextField({
    color:'#000',
    height:40,
    top:160,
	left:40, 
	right:40,
    hintText:'Search events',
    borderStyle:Titanium.UI.INPUT_BORDERSTYLE_ROUNDED,
    borderRadius:15,
    clearButtonMode:Ti.UI.INPUT_BUTTONMODE_ONFOCUS,
    keyboardToolbarColor: '#999',   
    keyboardToolbarHeight: 40,
    leftButton:magnifyingGlass
});
listWin.add(miniSearchBox);

magnifyingGlass.addEventListener('click', function(event) { miniSearchBox.focus(); });

miniSearchBox.addEventListener('return', function(event) {
	loadSearchResultsWindow(event.source.value);
});

var navItems = [];
navItems.push(makeRow(1, 'date', imagePath('menu-date-sm-iPad.png'), 65, 45, loadViewByDateBaseWindow));
navItems.push(makeRow(2, 'location', imagePath('menu-location-sm-iPad.png'), 65, 45, loadViewByLocationBaseWindow));
navItems.push(makeRow(3, 'category', imagePath('menu-category-sm-iPad.png'), 65, 45, loadViewByCategoryBaseWindow));
navItems.push(makeRow(4, 'random', imagePath('menu-random-sm-iPad.png'), 65, 45, loadRandomBaseWindow));
navItems.push(makeRow(5, 'suggest an event', imagePath('ico_suggest.png'), 38, 45, loadSuggestBaseWindow));

var rowHeight = 60;
var tableHeight = (navItems.length) * rowHeight;

var tableView =  Titanium.UI.createTableView({
	top:240,
	left:-1,
	right:-1,
	data:navItems,
    scrollable:false,
    height:tableHeight,
    rowHeight:rowHeight,
    backgroundColor:'transparent',
    borderColor:'#04506E',
    separatorColor:'#04506E'  
});
listWin.add(tableView);

var infoView = new ImageView({top:680, left:40, height:36, width:30, backgroundImage:'images/ico_information.png'});
listWin.add(infoView);
$(infoView).click(function(event){
	loadInfoBaseWindow();
});

function loadViewByDateBaseWindow() {		
	setColors('#bf5f16', '#7c3e03', 'red');

	clearFilters();
	clearPrimaryFilter();

	resetSelectedDateForSection('date');

	var newWindow = new Window(mixinWindowOptions({
	    id: 'win_suggest_event',
	    url: 'screens/view_events_list.js',
		title: 'Event',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
		showCategoryFilter:true
		
	}));
	newWindow.backButtonTitle = "";

	unLoadAllDetailWindows();
	homeWindow.visible = false;
	loadWindow(newWindow, null);
	splitWin.visible = true;	
	
}

function loadViewByCategoryBaseWindow() {	
	clearFilters(); // Need to clear any existing region/council selected
	resetSelectedDateForSection('category');
	loadBaseWindow('view_by_category', 'View by category', 'screens/filter_by_category.js', '#6cb640', '#4c8608','green');
}

function loadViewByLocationBaseWindow() {	
	loadBaseWindow('view_by_location', 'View by location', 'screens/filter_by_location.js', '#f0af13', '#ca8d00','orange');
}

function loadRandomBaseWindow() {	
	setColors('#009dc9', '#044456','blue');

	resetSelectedDateForSection('random');

	var newWindow = new Window(mixinWindowOptions({
	    id: 'win_suggest_event',
	    url: 'screens/event.js',
		title: 'Event',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
		showShakerBar:true,
		eventId: 'random',
	}));
	newWindow.backButtonTitle = "";

	unLoadAllDetailWindows();
	homeWindow.visible = false;
	loadWindow(newWindow, null);
	splitWin.visible = true;
}

function loadSuggestBaseWindow() {
	loadBaseWindow('win_suggest_an_event', 'Suggest an event', 'screens/suggest_event.js', '#00AF9D', '#027266','#014A42');
}

function loadInfoBaseWindow() {
	loadBaseWindow('win_info', 'About', 'screens/about.js', '#009dc9', '#044456','blue');
}

function goBackToHome() {
	splitWin.visible = false;
	homeWindow.visible = true;
}

function loadSearchResultsWindow(searchQuery) {
	
	if(emptyString(searchQuery))
	{
		messageBox(L('search_query_empty'));
		return;
	}
	
	setColors('#009dc9', '#044456','blue');

	var newWindow = new Window(mixinWindowOptions({
	    id: 'search_results',
	    url: 'screens/view_search_results.js',
		title: 'Results',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
		searchQuery:searchQuery
	}));
	newWindow.backButtonTitle = "";

	unLoadAllDetailWindows();
	homeWindow.visible = false;
	loadWindow(newWindow, null);
	splitWin.visible = true;
	
}

function loadBaseWindow(winId, winName, winURL, colour1, colour2, colour3) {
	
	setColors(colour1, colour2, colour3);

	var newWindow = new Window(mixinWindowOptions({
	    id: winId,
	    url: winURL,
		title: winName,
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR)
	}));
	newWindow.backButtonTitle = "";

	unLoadAllDetailWindows();
	homeWindow.visible = false;

	loadWindow(newWindow, null);

	splitWin.visible = true;
}

function unLoadAllDetailWindows() {
	Ti.API.debug(">>>>> Unloading all windows: ");
	var windows = Ti.App.loadedWindows;
	
	Ti.API.debug("There are " + windows.length + " windows to unload");
	//Needs to turn off animation while closing window from the nav group
	//o/w you will get the navbar is corrupted error message bcoz it is trying
	//to do nested animation which is not permitted in iOS.  Also need to
	//explicit calls windows.pop to reclaim memory. 
	//Setting Ti.App.loadedWindows to [] won't do it.
	for (i = 0, j = windows.length; i < j; i++) {
	var winToRemove = windows.pop();
	Ti.API.debug(">>>>> Unloading window: " + winToRemove.toString);
	detailNavGroup.close(winToRemove, {animated: false});
	winToRemove = null;
	}
	Ti.API.debug("After unload, there are " + windows.length + " windows");
	Ti.App.loadedWindows = windows;
}


// Load a new root window as selected from master view.
Ti.App.addEventListener('app:loadWindowIntoSplitView', function(e) {
	Ti.API.trace('alwisv 1');
	var winToOpen = getLastWindowOnSplitViewStack()
	Ti.API.trace('alwisv 2');
	Ti.API.debug(">>>>> Loading window: " + winToOpen);
	Ti.API.trace('alwisv 3');
	detailNavGroup.open(winToOpen);
	Ti.API.trace('alwisv 4');
});

// Load a new root window as selected from master view.
Ti.App.addEventListener('app:unloadWindowFromSplitView', function(e) {
	Ti.API.trace('auwfsv 1');
	var windows = Ti.App.loadedWindows;
	Ti.API.trace('auwfsv 2');
	var winToRemove = windows.pop();
	Ti.API.trace('auwfsv 3');
	Ti.API.debug(">>>>> Going to unload window: " + winToRemove);
	Ti.API.trace('auwfsv 4');
	//var winToRemove = getLastWindowOnSplitViewStack();
	Ti.API.trace('auwfsv 5');
	detailNavGroup.close(winToRemove, {animated: false});
	Ti.API.trace('auwfsv 6');
	winToRemove = null;
	Ti.API.trace('auwfsv 7');
	Ti.App.loadedWindows = windows;
	Ti.API.trace('auwfsv 8');
});