/*
 * ATTENTION: Some efforts has been put in order to produce this code.
 *            If you like and use it consider making a dontation in order
 *            to allow me to do more and provide you with more solutions.
 *
 *            Thanks,
 *            David Riccitelli
 *
 *            To donate, copy and paste this link in your browser:
 * https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=T5HUU4J5EQTJU&lc=IT&item_name=OAuth%20Adapter&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donate_LG%2egif%3aNonHosted
 *
 * Copyright 2010 David Riccitelli, Interact SpA
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * This library currently works only with Twitter, although I'd like to
 * spend some more time to make it generally compatible with other services
 * too.
 *
 * Sample use with Twitter:

 // create a new OAuthAdapter instance by passing by your consumer data and signature method
 var oAuthAdapter = new OAuthAdapter(
 'your-consumer-secret',
 'your-consumer-key',
 'HMAC-SHA1');

 // load the access token for the service (if previously saved)
 oAuthAdapter.loadAccessToken('twitter');

 // consume a service API - in this case the status update by Twitter
 oAuthAdapter.send('https://api.twitter.com/1/statuses/update.json', [['status','Hey @ziodave, I managed to use the #oauth adapter for @titanium consuming @twitterapi']],'Twitter','Tweet published.','Tweet not published.');

 // if the client is not authorized, ask for authorization. the previous tweet will be sent automatically after authorization
 if (oAuthAdapter.isAuthorized() == false)
 {
	 // this function will be called as soon as the application is authorized
     var receivePin = function() {
		 // get the access token with the provided pin/oauth_verifier
         oAuthAdapter.getAccessToken('https://api.twitter.com/oauth/access_token');
		 // save the access token
         oAuthAdapter.saveAccessToken('twitter');
     };

	 // show the authorization UI and call back the receive PIN function
     oAuthAdapter.showAuthorizeUI('https://api.twitter.com/oauth/authorize?' + oAuthAdapter.getRequestToken('https://api.twitter.com/oauth/request_token'), receivePin);
 }

 */
/*
 * The Adapter needs 2 external libraries (oauth.js, sha1.js) hosted at
 *  http://oauth.googlecode.com/svn/code/javascript/
 *
 * Save them locally in a lib subfolder
 */
//Ti.include('/includes/sha1.js');
//Ti.include('/includes/oauth.js');

// create an OAuthAdapter instance
var OAuthAdapter = function(pConsumerSecret, pConsumerKey, pSignatureMethod)
 {
	
	Ti.API.info('*********************************************');
	Ti.API.info('If you like the OAuth Adapter, consider donating at');
	Ti.API.info('https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=T5HUU4J5EQTJU&lc=IT&item_name=OAuth%20Adapter&currency_code=USD&bn=PP%2dDonationsBF%3abtn_donate_LG%2egif%3aNonHosted');
	Ti.API.info('*********************************************');	

    // will hold the consumer secret and consumer key as provided by the caller
    var consumerSecret = pConsumerSecret;
    var consumerKey = pConsumerKey;

    // will set the signature method as set by the caller
    var signatureMethod = pSignatureMethod;

    // the pin or oauth_verifier returned by the authorization process window
    var pin = null;

    // will hold the request token and access token returned by the service
    var requestToken = null;
    var requestTokenSecret = null;
    var requestUsername = null;
    var accessToken = null;
    var accessTokenSecret = null;
    var screenName = null;
    
    this.getScreenName = function(){ return screenName; };

    // the accessor is used when communicating with the OAuth libraries to sign the messages
    var accessor = {
        consumerSecret: consumerSecret,
        tokenSecret: ''
    };

    // holds actions to perform
    var actionsQueue = [];

    // will hold UI components
    var window = null;
    var view = null;
    var webView = null;
    var receivePinCallback = null;

	var deleteAccessToken = function(pService)
	{
        Ti.API.debug('Deleting access token for service [' + pService + '].');

        var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
        if (file.exists())
        	file.deleteFile();
        
		requestToken = null;
		requestTokenSecret = null;
		requestUsername = null;
		accessToken = null;
		accessTokenSecret = null;
		screenName = null;
		pin = null;
		
	};
	this.deleteAccessToken = deleteAccessToken;

    this.loadAccessToken = function(pService)
    {
        Ti.API.debug('Loading access token for service [' + pService + '].');

        var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
        if (file.exists() == false) return;

        var contents = file.read();
        if (contents == null) return;

        try
        {
            var config = JSON.parse(contents.text);
        }
        catch(ex)
        {
            return;
        }
        if (config.accessToken) accessToken = config.accessToken;
        if (config.accessTokenSecret) accessTokenSecret = config.accessTokenSecret;
        if (config.screenName) screenName = config.screenName;

        Ti.API.debug('Loading access token: done [accessToken:' + accessToken + '][accessTokenSecret:' + accessTokenSecret + '][screenName:' + screenName + '].');
    };
    this.saveAccessToken = function(pService)
    {
        Ti.API.debug('Saving access token [' + pService + '].');
        var file = Ti.Filesystem.getFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
        if (file == null) file = Ti.Filesystem.createFile(Ti.Filesystem.applicationDataDirectory, pService + '.config');
        file.write(JSON.stringify(
        {
            accessToken: accessToken,
            accessTokenSecret: accessTokenSecret,
            screenName: screenName,
        }
        ));
        Ti.API.debug('Saving access token: done.');
    };

    // will tell if the consumer is authorized
    this.isAuthorized = function()
    {
        return ! (accessToken == null || accessTokenSecret == null);
    };

    // creates a message to send to the service
    var createMessage = function(pUrl)
    {
        var message = {
            action: pUrl
            ,
            method: 'POST'
            ,
            parameters: []
        };
        message.parameters.push(['oauth_consumer_key', consumerKey]);
        message.parameters.push(['oauth_signature_method', signatureMethod]);
        return message;
    };

    // returns the pin
    this.getPin = function() {
        return pin;
    };
    
    // requests a requet token with the given Url
    this.getRequestTokenAsync = function(pUrl, args1, callback)
    {
        accessor.tokenSecret = '';

        var message = createMessage(pUrl);
        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);

        var client = Ti.Network.createHTTPClient();
        client.onerror = function() { callback(false, null); };
		client.onreadystatechange = function() {
			try{
				Ti.API.trace('token request ready state change: ' + this.readyState);
				if(this.readyState == 4)
				{
			        var responseParams = OAuth.getParameterMap(client.responseText);
			        requestToken = responseParams['oauth_token'];
			        requestTokenSecret = responseParams['oauth_token_secret'];
			        requestUsername = responseParams['screen_name'];
			        
			
			        Ti.API.debug('request token got the following response: ' + client.responseText);
					callback(true, this.responseText);
				}
	        }
	        catch(ex){
	        	Ti.API.warn('token request ready state change exception: ' + JSON.stringify(ex));
	        	callback(false, null);
	        }
	    };

        client.open('POST', pUrl, false);
        client.setRequestHeader("X-Requested-With", "XMLHttpRequest");

		var parameters = OAuth.getParameterMap(message.parameters);
        Ti.API.debug('getRequestTokenAsync url: ' + pUrl);
        Ti.API.debug('getRequestTokenAsync parameters: ' + JSON.stringify(parameters));
        client.send(parameters);
    }

    // unloads the UI used to have the user authorize the application
    var destroyAuthorizeUI = function()
    {
        Ti.API.debug('destroyAuthorizeUI');
        // if the window doesn't exist, exit
        if (window == null) return;
        
        webView.hide();

        // remove the UI
        try
        {
	        Ti.API.debug('destroyAuthorizeUI:webView.removeEventListener');
            webView.removeEventListener('load', authorizeUICallback);
	        Ti.API.debug('destroyAuthorizeUI:window.close()');
	        
	        // This needs to be delayed since the saveToken call
	        // is delayed 100ms and can potentially fire after the
	        // App event is fired and processed
            setTimeout(function(){ Ti.App.fireEvent('oauth:authorizationComplete') }, 200);
            
            window.close();
			// 	        Ti.API.debug('destroyAuthorizeUI:window.remove(view)');
			// window.remove(view);
			// 	        Ti.API.debug('destroyAuthorizeUI:view.remove(webView)');
			// 	        view.remove(webView);
			// 	        Ti.API.debug('destroyAuthorizeUI:nullifying');
			// 	        webView = null;
			//             view = null;
			//             window = null;
        }
        catch(ex)
        {
            Ti.API.debug('Cannot destroy the authorize UI. Ignoring.');
        }
    };

    // looks for the PIN everytime the user clicks on the WebView to authorize the APP
    // currently works with TWITTER
    // MT: Updated with changes by cyblob from the download page to fix
    // a breaking change made around May 30th
    var authorizeUICallback = function(e) {
        Ti.API.debug('authorizeUILoaded');
        Ti.API.debug('Url:'+ e.url );
        
        if(e.url.indexOf('authorize?denied') != -1) {
        	destroyAuthorizeUI();
        }
        else if( e.url.indexOf( 'https://api.twitter.com/oauth/authorize?' ) == -1 ) {
        	
        	var html = e.source.html || e.source.evalJS("'<html>'+document.getElementsByTagName('html')[0].innerHTML+'</html>'");
        	
            //Ti.API.debug('HTML:' + e.source.html);
            var positionStart = html.indexOf( '<code>' );
            var positionEnd = html.indexOf( '</code>' );
            if( positionStart != -1 && positionEnd != -1) {
                pin = html.substring( positionStart + 6, positionEnd );
                Ti.API.debug( "pin: " + pin );
                if (receivePinCallback)
                    setTimeout(receivePinCallback, 100);
                
                destroyAuthorizeUI();
                
            }
            nodeList = null;
            xmlDocument = null;
        }
    };

    // shows the authorization UI
    this.showAuthorizeUI = function(pUrl, pReceivePinCallback)
    {
        receivePinCallback = pReceivePinCallback;

/*
 * 
 * 


var headingContainer = new View({
	width:'100%',
	height: 40,
	backgroundColor: '#eee',
});

var heading = new Label({
	text: win().title,
	left: 25,
	top: 6,
	backgroundColor: 'transparent',
	font: { fontWeight: 'bold', },
	color: app.textColor,
});

var closeArea = new View({
	width: 35,
	height: 35,
	top: 0,
	right: 0,
});

var closeImage = new View({
	left:5,
	bottom: 5,
	width: 15,
	height: 15,
	backgroundImage: imagePath('close-gray.png'),
});

closeArea.add(closeImage);

headingContainer.add(heading);
headingContainer.add(closeArea);
window.add(headingContainer);
 * 
 */

		var cancelButton = Ti.UI.createButton({
			title: 'Cancel',
		});

		window = Titanium.UI.createWindow({
			backgroundColor:'#fff',
			fullscreen:false,
			modal:true,
			title: 'Twitter authorization',
			
			rightNavButton: cancelButton,
		});

        var transform = Ti.UI.create2DMatrix().scale(0);
        view = Ti.UI.createView({
        	top:0,
            width: getScreenWidth(),
            height: getWindowHeight(window),
            backgroundColor: 'white',
        });
        window.open();

        webView = Ti.UI.createWebView({
            url: pUrl,
			autoDetect:[Ti.UI.AUTODETECT_NONE]
        });
		Ti.API.debug('Setting:['+Ti.UI.AUTODETECT_NONE+']');
        webView.addEventListener('load', authorizeUICallback);
        view.add(webView);

        cancelButton.addEventListener('click', destroyAuthorizeUI);

        window.add(view);

        var animation = Ti.UI.createAnimation();
        animation.transform = Ti.UI.create2DMatrix();
        animation.duration = 500;
        view.animate(animation);
    };

    this.getAccessToken = function(pUrl)
    {
        accessor.tokenSecret = requestTokenSecret;

        var message = createMessage(pUrl);
        message.parameters.push(['oauth_token', requestToken]);
        message.parameters.push(['oauth_verifier', pin]);

        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);

        var parameterMap = OAuth.getParameterMap(message.parameters);
        for (var p in parameterMap)
        Ti.API.debug(p + ': ' + parameterMap[p]);

        var client = Ti.Network.createHTTPClient();
        client.open('POST', pUrl, false);
        client.send(parameterMap);

        var responseParams = OAuth.getParameterMap(client.responseText);
        accessToken = responseParams['oauth_token'];
        accessTokenSecret = responseParams['oauth_token_secret'];
        screenName = responseParams['screen_name'];

        Ti.API.debug('*** get access token, Response: ' + client.responseText);

        processQueue();

        return client.responseText;

    };

    var processQueue = function()
    {
        Ti.API.debug('Processing queue.');
        while ((q = actionsQueue.shift()) != null)
        send(q.url, q.parameters, q.title, q.successMessage, q.errorMessage);

        Ti.API.debug('Processing queue: done.');
    };

    // TODO: remove this on a separate Twitter library
    var send = function(pUrl, pParameters, pTitle, pSuccessMessage, pErrorMessage)
    {
        Ti.API.debug('Sending a message to the service at [' + pUrl + '] with the following params: ' + JSON.stringify(pParameters));

        if (accessToken == null || accessTokenSecret == null)
        {

            Ti.API.debug('The send status cannot be processed as the client doesn\'t have an access token. The status update will be sent as soon as the client has an access token.');

            actionsQueue.push({
                url: pUrl,
                parameters: pParameters,
                title: pTitle,
                successMessage: pSuccessMessage,
                errorMessage: pErrorMessage,
            });
            return;
        }

        accessor.tokenSecret = accessTokenSecret;

        var message = createMessage(pUrl);
        message.parameters.push(['oauth_token', accessToken]);
        for (p in pParameters) message.parameters.push(pParameters[p]);
        OAuth.setTimestampAndNonce(message);
        OAuth.SignatureMethod.sign(message, accessor);

        var parameterMap = OAuth.getParameterMap(message.parameters);
        for (var p in parameterMap)
        Ti.API.debug(p + ': ' + parameterMap[p]);

        var client = Ti.Network.createHTTPClient();
        
        client.open('POST', pUrl, false);
        client.send(parameterMap);
        
        if (client.status == 200) {
        	
        	if(typeof pSuccessMessage == 'function')
			{
				pSuccessMessage(client.responseText);
			}
        	else
        	{
	            Ti.UI.createAlertDialog({
	                title: pTitle,
	                message: pSuccessMessage
	            }).show();
            }
        } else {
        	if(typeof pErrorMessage == 'function')
        	{
        		pErrorMessage(client.responseText);
        	}
        	else
        	{
	            Ti.UI.createAlertDialog({
	                title: pTitle,
	                message: pErrorMessage
	            }).show();
	        }
        }

        Ti.API.debug('*** sendStatus, Response: [' + client.status + '] ' + client.responseText);

        return client.responseText;

    };
    this.send = send;

};
