// Global downloader shared amongst all remote functions. 
// !IMPORTANT - Android has leak issues with HTTP. DONT USE INDIVIDUAL/MULTIPLE downloaders
var downloader = Ti.Network.createHTTPClient({timeout:10000});
var dataLoadingIndicator = null;

downloader.onerror = function(e){

	if(tableView != null)
	{
		var row = makeErrorRow(L("http_client_error"));
		tableView.setData([row]);
		if(isIOS()) 
			cancelLoadingIndicator();
		else
			dataLoadingIndicator.hide();
	}
};

if(isAndroid()) {
	
	downloader.onreadystatechange = function() {
		if(downloader.readyState == 1) {
			dataLoadingIndicator = Ti.UI.createActivityIndicator({
				width: 90,
				height: 90,
				message: 'Loading...'
			});

			dataLoadingIndicator.show();
		} else if (downloader.readyState == 4 && dataLoadingIndicator != null) {
			dataLoadingIndicator.hide();	
		}
	}
}
/*
 * @param url URL with no trailing ? or &
 * @param autoParams: an array of any combination of 'category', 'region', 'council'
 * @param extraParams: an object of key->value pairs to be passed through verbatim, EXCEPT 'date' which is treated as a Date object and converted to yyyy-mm-dd
 */
function buildUrl(url, autoParams, extraParams)
{
	var args = [];
	
	if(autoParams != null)
	{
		for(var i in autoParams)
		{
			switch(autoParams[i])
			{
				case 'category':
					if(isCategorySelected()) {
						args.push("category=" + getSelectedCategory().id);
					}
					break;
	
				case 'council':
					var council = getCouncilFilter();
					if(council != null)
						args.push('council=' + council.id);
					break;
					
				case 'region':
					var region = getRegionFilter();
					if(region != null)
						args.push("region=" + region.id);
					break;	
					
				default:
					alert('WARNING: Unrecognised parameter passed to buildUrl: ' + params[i]);
					break;
			}
		}
	}
	
	if(extraParams != null)
	{
		for(var i in extraParams)
		{
			// A "date" parameter will be an actual Date
			if(i == 'date')
			{
				var date = extraParams[i];
				if(date != null)
				{
					var d = stringifyDate(date);
					if(d != null)
					{
						args.push("date=" + d);
					}
				}
			}
			else
			{
				args.push(i + '=' + extraParams[i]);
			}
		}
	}
	
	var fullUrl = appProps.API_URL + url;
	
	if(args.length > 0)
	{
		fullUrl += '?' + args.join('&');
	}
	
	Ti.API.debug('Built URL: ' + fullUrl);
	
	return fullUrl;	
}

function loadRemoteCategories(tableView, selectedDate, isFullList) {

	downloader.open("GET", buildUrl('/categories.json' + (isFullList ? '?counts=false' : ''), null, { date: selectedDate }));	  

	downloader.onload = function() { 
		var rowData = [];
		var categories = JSON.parse(this.responseText);
		for (var i = 0; i < categories.length; i++) {
			var row = makeLinkedStyledRow(categories[i].id, categories[i].title, categories[i].event_count, null, viewEventsForCategory, i > 0);
			rowData.push(row);
		}
		tableView.setData(rowData);
		
		finishingTick();
		
		cancelLoadingIndicator();
		Ti.API.trace('finished loadRemoteCategories');
	};  

	downloader.send();
}

function loadRemoteMonthCounts(tableView) {

	downloader.open("GET", buildUrl('/monthcounts.json', ['category','region','council']));	  

	downloader.onload = function() { 
		var rows = [];
		
		var currentDate = new Date();
		var currentMonth = currentDate.getMonth(), currentYear = currentDate.getYear();
		
		var selectedIndex = -1;
		
		var data = JSON.parse(this.responseText);
		
		if(handleError(data, rows, 'No matching events were found.'))
		{
			// Error row(s) will have been stored in the rows array
		}
		else
		{
			for(var i = 0; i < data.length; i++)
			{
				var yearData = data[i];
				rows.push(makeDividerRow(yearData.year.toString(), i>0));
	
				for(var j = 0; j < yearData.data.length; j++)
				{
					var monthData = yearData.data[j];
					
					var id = ((yearData.year * 10000) + (monthData.month * 100) + 1).toString();
					
					var text = monthNames[monthData.month - 1];
					var isSelectedRow = (yearData.year == currentDate.getFullYear() && monthData.month == (currentMonth + 1));
					if(isSelectedRow)
					{
						selectedIndex = rows.length;
						text = "This month, " + text;
					}
					var row = makeLinkedStyledRow(id, text, monthData.count.toString(), null, loadEventsWindowForMonth, true);
					
					if(isSelectedRow)
					{
						row.backgroundColor = app.backgroundBlueColor;
					}

					rows.push(row);
				}
			}
		}
		tableView.setData(rows);
		
		cancelLoadingIndicator();
		if(selectedIndex != -1)
		{
			scrollTableView(tableView, selectedIndex);
		}
		
	};

	downloader.send();
}

function loadRemoteCouncils(tableView, isFullList) {
	
	downloader.open("GET", buildUrl('/councils.json' + (isFullList ? '?counts=false' : '')));

	downloader.onload = function() { 
		var rowData = [];  
		var data = eval('('+this.responseText+')');
		for (var i = 0; i < data.length; i++) { 
			rowData.push(makeLinkedStyledRow(parseInt(data[i].id, 10), data[i].title, data[i].event_count, null, viewEventsForCouncil, i > 0));
		}
		tableView.data = rowData;

		cancelLoadingIndicator();
	};  

	downloader.send();	
}

function loadRemoteRegions(tableView, isFullList) {

	downloader.open("GET", buildUrl('/regions.json' + (isFullList ? '?counts=false' : '')));
		
	downloader.onload = function() { 
		var rowData = [];  
		var data = eval('('+this.responseText+')');
		for (var i = 0; i < data.length; i++) { 
			rowData.push(makeLinkedStyledRow(data[i].id, data[i].title, data[i].event_count, null, viewEventsForRegion, i > 0));
		}
		tableView.data = rowData;
		
		cancelLoadingIndicator();
	};
	
	downloader.send();	
}

function handleError(data, rowOutputArray, emptyDataMessage)
{
	if(data.errors != null)
	{
		Ti.API.debug('handleError: Error message(s) found in response!!')
		for(var i in data.errors)
		{
			Ti.API.debug("Error message: " + JSON.stringify(data.errors[i]));
			rowOutputArray.push(makeErrorRow(data.errors[i]));
		}

		cancelLoadingIndicator();
		
		return true;
	}
	else if(emptyDataMessage != null && data.length == 0)
	{
		Ti.API.debug('handleError: No rows found. Using the empty data message');
		rowOutputArray.push(makeErrorRow(emptyDataMessage));
		cancelLoadingIndicator();
		
		return true;
	}
	else
	{
		Ti.API.debug('handleError: No error messages found in response')
		return false;
	}
}


/*
 * @param month The Javascript month index (0 - 11)
 */
function loadRemoteMonth(tableView, month, year, includeAllMonth) {
	
	var extraArgs = {};
	if(month != null) { extraArgs.month = (month+1); }
	if(year != null) { extraArgs.year = year; }	

	downloader.open("GET", buildUrl('/monthlyevents.json', ['council', 'region', 'category'], extraArgs));  

	downloader.onload = function() { 
		var rowData = [];  
		var data = JSON.parse(this.responseText);
		
		Ti.API.debug("loadRemoteMonth() received data: " + JSON.stringify(data));
		
		if(handleError(data, rowData, L('months_no_events_returned')))
		{
			// rowData will have been populated with any error message(s)
		}
		else
		{
			var hadMonth = false;
			
			if(includeAllMonth)
			{
				hadMonth = true;
				rowData.push(makeLinkedStyledRow(new Date(year,month,1), "All month", "", null, chooseAllMonth));
			}
		
			var currentDate = new Date();
			var currentMonth = currentDate.getMonth(), currentYear = currentDate.getYear(), currentDay = currentDate.getDate();
			
			var isCurrentMonthAndYear = (currentMonth === month && currentDate.getFullYear() === year);
			
			var selectedIndex = -1;
						
			for (var i = 0; i < data.length; i++) { 
				var rowDate = new Date();
				rowDate.setDate(data[i].id);
				if(month!= null) { rowDate.setMonth(month); }
				if(year!= null) { rowDate.setFullYear(year); }
				var rowDateStringy = rowDate.getDate().toString() + " " + getMonthName(rowDate.getMonth()) + ", " + getDayName(rowDate.getDay());
				var isSelectedRow = (isCurrentMonthAndYear && data[i].id == currentDay);
				
				var row = makeLinkedStyledRow(rowDate.getTime(), (isSelectedRow ? 'Today, ' : '') + rowDateStringy, data[i].event_count.toString(), null, chooseDate, hadMonth || i > 0);
					
				if(isSelectedRow)
				{
					row.backgroundColor = app.backgroundBlueColor;
					selectedIndex = rowData.length;
				}
				rowData.push(row);
			}
		}
		tableView.setData(rowData);
		
		cancelLoadingIndicator();

		if(selectedIndex != -1)
		{
			scrollTableView(tableView, selectedIndex);
		}

	};  

	downloader.send();
}

function loadSearchResults(query, callback, num)
{
	if(num == null)
		num = 20;
	
	downloader.open("GET", buildUrl("/events/search.json", null, { q: escape(query), num: num }));  

	downloader.onload = function() {
		var data = JSON.parse(this.responseText);

		cancelLoadingIndicator();
		
		callback(data);
	};  
	
	downloader.send();
}

/*
 * This function shows the details of an event, based on a click on a
 * row that has it's id property set to the id of the event to display.
 */
function showEventFromRowClick(event) 
{
	Ti.API.debug('showEventFromRowClick( ' + JSON.stringify(event) +  ')');
	viewEventWithId(event.source.id);
}

function populateEventsTable(tableView, fields, data, isSearch)
{
	Ti.API.debug('populateEventsTable() called');

	var section = new TableViewSection({});
	
	section.headerView = new View({width:100, height:1, color:'green',})
	
	var ongoingRows = [], regularRows = [];
	
	filteredEventLocations = [];

	if(!handleError(data, regularRows, null))
	{
		var showAllLink = false;
		
		Ti.API.debug('events data contains ' + data.events.length + ' elements');
		
		for (var i = 0, j = data.events.length; i < j; i++) {
		
			var newRow = makeSummaryRow(data.events[i], null, showEventFromRowClick, fields);
		
			if(data.events[i].date.start == data.events[i].date.end)
			{
				if(regularRows.length > 0)
					addFakeSeparator(newRow);
				regularRows.push(newRow);
			}
			else
			{
				addFakeSeparator(newRow);
				ongoingRows.push(newRow);
			}

			if((data.events[i].coordinates.lat != 0) && (data.events[i].coordinates.lng != 0)) {
				filteredEventLocations.push({
					lat:data.events[i].coordinates.lat, 
					lng:data.events[i].coordinates.lng,
					title:data.events[i].title,
					eventId:data.events[i].id,
					subtitle: getEventLocation(data.events[i]),
				});	
			}
		
		}
		
		// If there are more events to load then add the "view all" footer
		if(i < data.total_results)
		{
			showAllLink = true;
		}
	}
	
	if(ongoingRows.length > 0)
	{
		var ongoingHeaderRow = makeDividerRow('Ongoing Events');
		
		if(regularRows.length > 0)
			addFakeSeparator(ongoingHeaderRow);
		
		regularRows.push(ongoingHeaderRow);
		
		for(var i in ongoingRows)
		{
			regularRows.push(ongoingRows[i]);
		}
	}
	
	if(showAllLink)
	{
		var viewAllRow = makeViewAllEventsRow();
		viewAllRow.addEventListener('click', function() {
			if(isSearch)
			{
				loadSearchResults(win().searchQuery, populateSearchResults, 200);
			}
			else
			{
				loadRemoteEventsWithFilters(tableView, fields, true, (win().isCalendarMonth ? true : false));
			}
		});
		regularRows.push(viewAllRow);
	}
	
	// Configure the map button in the top right corner
	if(isIOS()) {
		setNavBarMap(filteredEventLocations.length > 0, true, clickMapButtonMultiple);
	} else {
		makeAndroidMapFooter(filteredEventLocations.length > 0, tableView, clickMapButtonMultiple);
	}
	
	tableView.setData(regularRows);
	
	cancelLoadingIndicator();
}

function makeDividerRow(title, includeSeparator)
{
	var ongoingHeaderRow = Ti.UI.createTableViewRow({ 
		height: 35,
		backgroundColor: app.backgroundBlueColor
	});
	
	if(isIOS()) {
		ongoingHeaderRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
	}
	
	var label = Ti.UI.createLabel({
		text:title,
		color: '#333',
		font: { fontSize: 13, fontWeight: 'bold', },
		textAlign: 'center',
	});
	ongoingHeaderRow.add(label);
	
	if(includeSeparator === true)
		addFakeSeparator(ongoingHeaderRow);
	
	return ongoingHeaderRow;
}

function loadRemoteEventsWithFilters(tableView, fields, viewAll, isWholeMonth) {
	var extraArgs = {
		date: getSelectedDate(),
		num: (viewAll === true ? 200 : 10),
	};
	
	// TODO: Should be a cleaner way to do this
	if(win().title == 'Nearby')
	{
		var location = getLastLatLng();
		extraArgs.lat = location.lat;
		extraArgs.lng = location.lng;
	}

	if(isWholeMonth) {
		extraArgs.range = 'month';
	}
	
	downloader.open("GET", buildUrl('/events/filter.json', ['region', 'council', 'category'], extraArgs));  

	downloader.onload = function() {
		
		populateEventsTable(tableView, fields, JSON.parse(this.responseText), false);
	};  

	downloader.send();
}

/*
 * Loads the event with the specified event ID, unless it is the special
 * value 'random' - in which case a random event is loaded.
 */
function loadRemoteEvent(eventId) {
	
	var url;
	
	if(eventId == 'random')
		url = buildUrl('/events/random.json', ['category', 'council', 'region']);
	else
		url = buildUrl('/events/show.json', null, { id: eventId });

	Ti.API.info(">>> Loading an event with URL: " + url);

	downloader.open("GET",url);  

	downloader.onload = function() { 
		var data = eval('('+this.responseText+')');
		
		if(loadEventFromData != null)
		{
			loadEventFromData(data);
		}
		
		cancelLoadingIndicator();
	};  

	downloader.send();
}

function sendSuggestEvent(callback) {
	
	var postBody = {
		"title" : getSharedPropertyAsString(appProps.KEY_EVENT_NAME),
		"description" : getSharedPropertyAsString(appProps.KEY_EVENT_DESCRIPTION),
		"start_date" : getSharedPropertyAsString(appProps.KEY_EVENT_START_DATE),
		"end_date" : getSharedPropertyAsString(appProps.KEY_EVENT_END_DATE),
		"location" : getSharedPropertyAsString(appProps.KEY_EVENT_LOCATION),
		"street" : getSharedPropertyAsString(appProps.KEY_EVENT_STREET),
		"town" : getSharedPropertyAsString(appProps.KEY_EVENT_TOWN),
		"state" : getSharedPropertyAsString(appProps.KEY_EVENT_STATE),
		"message" : getSharedPropertyAsString(appProps.KEY_EVENT_MESSAGE),
		"council" : getSharedPropertyAsString(appProps.KEY_EVENT_COUNCIL),
		"region" : getSharedPropertyAsString(appProps.KEY_EVENT_REGION),
		"phone" : getSharedPropertyAsString(appProps.KEY_EVENT_PHONE),
		"email" : getSharedPropertyAsString(appProps.KEY_EVENT_EMAIL),
		"web" : getSharedPropertyAsString(appProps.KEY_EVENT_WEBSITE),
		"category1" : getSharedPropertyAsString(appProps.KEY_EVENT_CAT1),
		"category2" : getSharedPropertyAsString(appProps.KEY_EVENT_CAT2),
		"person_name" : getSharedPropertyAsString(appProps.KEY_EVENT_PERSON_NAME),
		"person_email" : getSharedPropertyAsString(appProps.KEY_EVENT_PERSON_EMAIL),
		"person_telephone" : getSharedPropertyAsString(appProps.KEY_EVENT_PERSON_PHONE)
	}

	url = appProps.API_URL + "/events";
	downloader.open("POST",url);
	//downloader.send(postBody);
	
	callback(true);
}

function cancelLoadingIndicator() {
	Ti.API.trace('cancelLoadingIndicator start');
	if(isAndroid()) {
		// Ti.App.fireEvent('app:stopActivityIndicator');
	}
	Ti.API.trace('cancelLoadingIndicator finish');
}
