//Ti.include('../redux.js');



Ti.include('/includes/sha1.js');
Ti.include('/includes/oauth.js');
Ti.include('/includes/oauth_adapter.js');

var sharing = {
	currentEvent: null,
	currentUsername: null,
	facebookUsername: null,
};


// Twitter API instructions only
var oAuthAdapter = new OAuthAdapter(
		'3khtJagdwtohzjxcVGEm4T4S4Z3zWZxwljGYbMcY4Y',
		'BtmFTUNFqfGbk8372ptuLg',
        'HMAC-SHA1');

// Load 3rd party modules
if (isIOS()) {
	var Calendar = require('com.ti.calendar');
}

function shareToCalendar(currentEvent)
{
	var evStartDate = parseDate(currentEvent.date.start);
	var evEndDate = parseDate(currentEvent.date.end);
	var message = 'This event has been added to your calendar.';
	if(isIOS()) {
		var ev = Calendar.createItem({
			title: currentEvent.title,
			location: getEventLocation(currentEvent),
			startDate: evStartDate,
			endDate: evEndDate,
		});
		
		var success = ev.saveEvent();
		message = (success
			? 'This event has been added to your calendar.'
			: 'Sorry, this event could not be added to your calendar at this time.');
	} else {

		try {
			// MT: Important! The type conversion that occurs on 
			// Ti.Android.Calendar.selectableCalendars when accessing it directly
			// throws an exception when it tries to retrieve the default value,
			// so only compare with null or access "known good" properties.
			if(Ti.Android.Calendar.selectableCalendars == null || 
				Ti.Android.Calendar.selectableCalendars.length == 0)
			{
				message = 'In order to save an event, you will need to set up a calendar in your device settings.';
			}
			else
			{
				Ti.API.info("Ti.Android.Calendar.selectableCalendars is " + Ti.Android.Calendar.selectableCalendars.length);
				var calendar = Ti.Android.Calendar.selectableCalendars[0];
				
				var details = {
				    title: currentEvent.title,
				    begin: evStartDate,
				    end: evEndDate
				};
				
				var event = calendar.createEvent(details);
			}
		} catch(e) {
			Ti.API.debug('Adding event to calendar threw an exception: ' + JSON.stringify(e));
			message = 'Sorry, this event could not be added to your calendar at this time.';
		}
	}

	var alertDialog = Ti.UI.createAlertDialog({
		message: message,
		buttonNames: ['OK'],
	});
	alertDialog.show();
}

function shareToFacebook(currentEvent) {

	setShakeEventsEnabled(false, 'facebook share started');
	
	awaitingFacebookPost = true;

	showLoadingWindow();

	if(Ti.Facebook.loggedIn)
	{
		handleFacebookLoginChange();
	}
	else
	{
		Ti.Facebook.authorize();
	}

}



function showEditingDialog(title, characterLimit, currentEvent, postCallback, logoutCallback, shareTemplateName)
{
	var closeCallback = function() { setShakeEventsEnabled(true, 'editing dialog closed'); };
	
	var options = {
		backgroundColor:'#fff',
		layout: 'vertical',
		url: (isAndroid() ? 'screens/' : '') + 'edit_message.js',
		title: title,
		characterLimit: characterLimit,
		currentEvent: currentEvent,
		postCallback: postCallback,
		oAuthAdapter: oAuthAdapter,
		currentUsername: sharing.currentUsername,
		logoutCallback: logoutCallback,
		shareTemplateName: shareTemplateName,
	};
	
	if(isIOS())
	{	
		options.height = 400;
		options.width = 300;
		options.borderWidth = 8;
		options.borderColor = '#999';
		options.borderRadius = 10;
	}
	else
	{
		options.fullscreen = true;
	}
	
	var w = Titanium.UI.createWindow(options);
	w.addEventListener('close', closeCallback);
	bounceOpen(w);
}

function showTwitterEditingDialog(currentEvent)
{
	sharing.currentUsername = oAuthAdapter.getScreenName();
	showEditingDialog('Post to Twitter', 140, currentEvent, doTwitterPost, twitterLogout, 'twitter', setShakeEventsEnabled);
}

function showFacebookEditingDialog(currentEvent)
{
	sharing.currentEvent = currentEvent;
	
	// Look up the Facebook username if we haven't already
	if(sharing.facebookUsername == null)
	{
		getFacebookUserData(function(e) {
			if(e.success)
			{
				var data = JSON.parse(e.result);
				sharing.facebookUsername = data.name;
				sharing.currentUsername = data.name;
			}
			else
			{
				sharing.currentUsername = null;
			}
			
			showEditingDialog('Post to Facebook', 0, sharing.currentEvent, doFacebookPost, facebookLogout, 'facebook', setShakeEventsEnabled);
		});
	}
	else
	{
		sharing.currentUsername = sharing.facebookUsername;
		showEditingDialog('Post to Facebook', 0, sharing.currentEvent, doFacebookPost, facebookLogout, 'facebook', setShakeEventsEnabled);
	}
}

function facebookLogout()
{
	Titanium.Facebook.logout();
}

function twitterLogout()
{
	oAuthAdapter.deleteAccessToken('twitter');
}

function getTwitterDetails()
{
	
	oAuthAdapter.deleteAccessToken();
	
	// load the access token for the service (if previously saved)
	oAuthAdapter.loadAccessToken('twitter');
}

/*
 * @param message The message, already encoded (e.g. using encodeURI)
 */
function doTwitterPost(currentEvent, message, afterPostCallback)
{
	var tweetSuccess = function(){
		alert('Tweet posted!');
		afterPostCallback(true, null);
	};
	
	var tweetFailed = function(e) {
		
		var message = null;
		if(e != null)
		{
			var response = JSON.parse(e);
			if(response.error != null)
				message = response.error;
		}
		afterPostCallback(false, message);
	};
	
	oAuthAdapter.send('https://api.twitter.com/1/statuses/update.json', [['status', message]], 'Twitter', tweetSuccess, tweetFailed);
}

function getFacebookUserData(callback)
{
	Ti.Facebook.requestWithGraphPath('me', {}, 'GET', function(e)
		{
			Ti.API.debug('Titanium me GET returned: ' + JSON.stringify(e));
			callback(e);
		});
}

function doFacebookPost(currentEvent, message, afterPostCallback)
{
	var data = {
		
		// The following should be set based on the event data after the user
		// has made any changes to the message, etc.
		name: currentEvent.title,
		message: message,
		caption: currentEvent.location,
		description: currentEvent.body,
		link: 'http://www.vic.gov.au/',
		
		// For testing limit this post so that only I can see it
		privacy: {
			value: 'CUSTOM',
			friends: 'SELF'
		},
	};

	if(currentEvent.url != null && currentEvent.url != '')
	{
		data.link = currentEvent.url;
	}

	// Attach the first picture if any are present
	if(currentEvent.photos != null && currentEvent.photos.length > 0)
	{
		data.picture = currentEvent.photos[0].url;
	}
	
	Ti.API.debug('Facebook me/feed POST data: ' + JSON.stringify(data));
	
	Ti.Facebook.requestWithGraphPath('me/feed', data, 'POST', function(e)
		{
			Ti.API.debug('Titanium me/feed POST returned: ' + JSON.stringify(e));
			if(e.success)
			{
				afterPostCallback(true, null);
			}
			else
			{
				if(e.error)
				{
					afterPostCallback(false, e.error);
				}
				else
				{
					afterPostCallback(false, null);
				}
			}
		});
}

function twitterAuthorizationCallback()
{
	// Callback function 
	if(oAuthAdapter.isAuthorized())
	{
		showTwitterEditingDialog(sharing.currentEvent);
	}
	else
	{
		hideLoadingWindow();
	}
}

Ti.App.addEventListener('oauth:authorizationComplete', twitterAuthorizationCallback)

function shareToTwitter(currentEvent) {
	
	sharing.currentEvent = currentEvent;
	
	showLoadingWindow();
	
	// load the access token for the service (if previously saved)
	oAuthAdapter.loadAccessToken('twitter');
	
	// if the client is not authorized, ask for authorization. the previous tweet will be sent automatically after authorization
	if (oAuthAdapter.isAuthorized() == false)
	 {
	    oAuthAdapter.getRequestTokenAsync('http://api.twitter.com/oauth/request_token', [['oauth_callback', 'oob']],
	    	shareToTwitterCallback);
	    
	}
	else
	{
		showTwitterEditingDialog(currentEvent);
	}
}

function shareToTwitterCallback(success, token)
{
	Ti.API.trace('shareToTwitterCallback received success: ' + success + ', token: ' + token);
	
	if(success)
	{
	    // this function will be called as soon as the application is authorized
	    var receivePin = function() {
	        // get the access token with the provided pin/oauth_verifier
	        oAuthAdapter.getAccessToken('http://twitter.com/oauth/access_token');
	        // save the access token
	        oAuthAdapter.saveAccessToken('twitter');
			//showTwitterEditingDialog();
	    };
	    
	    // show the authorization UI and call back the receive PIN function
	    oAuthAdapter.showAuthorizeUI('http://twitter.com/oauth/authorize?' + token, receivePin);
	}
	else
	{
		messageBox('Sorry, could not authenticate with Twitter.');
		hideLoadingWindow();
	}
}
