// Application utility functions

// Created by: Jarrod Dumble
// Created at: 28 July 2011

function getTableContentWidth(window)
{
	if(isAndroid())
		return (getScreenWidth() - 3 * app.tablePadLeft);
	else if(isIPhone())
		return (getScreenWidth() - 4 * app.tablePadLeft);
	else if(isIPad())
		return (window.width - 4 * app.tablePadLeft);
		
}

function scrollTableView(tableView, selectedIndex)
{
	if(isIOS()) {
		tableView.scrollToIndex(selectedIndex, { position: Ti.UI.iPhone.TableViewScrollPosition.TOP, });
	} else {
		tableView.scrollToIndex(selectedIndex);
	}
}

function getStandardTableWidth()
{
	if(isIPad()) {
		return win().width * 1.08;
	} else if (isIPhone()){
		return getScreenWidth() * 0.98
	} else if (isAndroid()) {
		return getScreenWidth();
	}
}

var monthNames = [
    "January", "February", "March",
    "April", "May", "June",
    "July", "August", "September",
    "October", "November", "December"
];
var dayNames = [
	"Sunday",
	"Monday", "Tuesday", "Wednesday", 
	"Thursday", "Friday", "Saturday"
];

var latitude = 0, longitude = 0;

/*
 * Replacement for alert() call, with no title
 */
function messageBox(message)
{
	var dialog = Ti.UI.createAlertDialog({
		message: message,
		buttonNames: ['OK'],
	});
	dialog.show();
}

/**
 * Checks if we are online, and shows an alert if we are not.
 */
function checkInternet() {
    if (!Ti.Network.online && isIOS()) {
		showLoadingWindow();
	
		var w = Titanium.UI.createWindow({
			backgroundColor:'#eee',
			borderWidth:1,
			borderColor:'#eee',
			height:200,
			width:250,
			borderRadius:10,
		});
	
		var lbl = new Label({
			width: 200,
			top: 35,
			height: 'auto',
			backgroundColor: 'transparent',
			opacity: 1,
			textAlign:'center',
			text: 'You must have an\ninternet connection to\nuse Vic Events.',
		});
		
		w.add(lbl);
		
		var btn = new Button({
			width: 200,
			height: 40,
			bottom: 35,
			title: 'retry',
			font: { fontSize: 15, fontWeight: 'bold', },
			backgroundImage: imagePath('turquoise/btn-dark.png'),
		})
		
		w.add(btn);
		
		bounceOpen(w);
	
		btn.addEventListener('click', function() {
			if(!Ti.Network.online)
			{
				alert('Sorry, no internet connection was detected.');
			}
			else
			{
				hideLoadingWindow();
				var t3 = Titanium.UI.create2DMatrix().scale(0);
				w.close({transform:t3,duration:300});
			}
		})
	}
}

/*
 * Displays the window with a standard 'bounce' effect on iOS, or just displays
 * the window on Android.
 * @param window The window to open
 */
function bounceOpen(window)
{
	if(isIOS()) {

		var t = Titanium.UI.create2DMatrix().scale(0);
		window.transform = t;

		// Create first transform to go beyond normal size
		var t1 = Titanium.UI.create2DMatrix().scale(1.1);
		var t2 = Titanium.UI.create2DMatrix().scale(1.0);
	
		// Create an animation that runs the first transform, then the second
		var a = Titanium.UI.createAnimation({ transform: t1, duration: 200 });
		a.addEventListener('complete', function() { window.animate({transform:t2, duration:200}); });
		
		window.open(a);
	} else {
		window.open();
	}
}

/**
 *
 */
function isIOS() {
	return Titanium.Platform.osname == "iphone" || Titanium.Platform.osname == "ipad";
}

/**
 *
 */
function isIPhone() {
	return Titanium.Platform.osname == "iphone";
}

/**
 *
 */
function isIPad() {
	return Titanium.Platform.osname == "ipad";
}

/**
 *
 */
function isAndroid() {
	return Titanium.Platform.osname == "android";
}

/**
 * Formats a date, returning a nice string that can be shown to the user.
 * @param date The date to format; leave blank to use the current time!
 */
function formatDate(date) {
    date = date || new Date();
    var result = date.getDate() + '/' + (date.getMonth() + 1) + "/" + date.getFullYear();
    if (date.getHours() + date.getMinutes() + date.getSeconds() != 0) {
        result += " " + date.getHours() + ":" + date.getMinutes() + ":" + date.getSeconds();
    }
    return result.replace(/\b(\d)\b/g, '0$1');
}

/**
 * Get the resources directory of the application
 */
function appRoot() {
	return Ti.Filesystem.resourcesDirectory;
}

/**
 * Using the Resources folder as a root, return an absolute path to the winURL
 * @param winURL The path to the JS file relative to the Resources folder
 */
function toAbsPath(winURL) {
	return appRoot() + winURL;
}

/**
 * Create and display a new window using the provided parameters
 * @param winId The ID to give the window
 * @param winURL The path to the JS file relative to the Resources folder
 * @param winTitle The title to put on the window
 */
function loadWindow(newWin, isFullScreen) {
	Ti.API.trace('loadWindow 1');
	if(isAndroid()) {
		newWin.orientationModes = [Ti.UI.PORTRAIT, Ti.UI.UPSIDE_PORTRAIT]; // Force Android to lock orientation to portrait, no other method is reliable.
		newWin.open();
	}
	else if(isIPhone())
	{
		if(isFullScreen)
		{
			Ti.API.trace('loadWindow 2');
			newWin.open();
		}
		else
		{
			Ti.API.trace('loadWindow 3');
			tab().open(newWin);
		}
	}
	else if(isIPad()) {
			Ti.API.trace('loadWindow 4');
		addWindowToSplitViewStack(newWin)
			Ti.API.trace('loadWindow 5');
		Ti.App.fireEvent('app:loadWindowIntoSplitView');
	}
}

// TODO: Can remove?
function closeWindow(win, transition)
{
	if(isAndroid()){
		win.close();
	}
	else
	{
		if(transition != null)
		{
			tab().close(win, {transition: transition})
		}
		else
		{
			tab().close(win);
		}
	}
}

function unloadCurrentWindow() {
	if(isIPad()) {
		Ti.App.fireEvent('app:unloadWindowFromSplitView');
	} else {
		win().close();
	}	
}

function addWindowToSplitViewStack(newWin) {
	// Add the window to the global list of windows loaded.
	Ti.API.trace('awtsvw 1');
	var windows = Ti.App.loadedWindows;
	Ti.API.trace('awtsvw 2');
	windows.push(newWin);
	Ti.App.loadedWindows = windows;
	Ti.API.trace('awtsvw 3');
}

function getLastWindowOnSplitViewStack() {
	try{
		return Ti.App.loadedWindows[Ti.App.loadedWindows.length - 1];
	}
	catch(e){
 		return null;
	}
}

function removeLastWindowOnSplitViewStack() {
	try{
		Ti.API.debug(">>>> Last window on stack is " + Ti.App.loadedWindows.length);
		// Add the window to the global list of windows loaded.
		var windows = Ti.App.loadedWindows;
		windows.pop();
		Ti.App.loadedWindows = windows;
		Ti.API.debug(">>>> Just removed window on stack is " + Ti.App.loadedWindows.length);
	}
	catch(e){
		Ti.API.debug(">>>> Error in removing last window from stack array: " + e.toString());
 		return null;
	}
}

/**
 * Create a path to an image.
 * @param image_file_name The path to the image rooted in the /images directory.
 */
function imagePath(image_file_name) {
	return appRoot() + 'images/' + image_file_name;
}

function coloredImagePath(image_file_name) {
	return imagePath( getSharedPropertyAsString(appProps.KEY_FILTER_COLOR_SCHEME) + '/' + image_file_name);
}

/**
 * Launch the web browser on the device
 */
function launchBrowser(url) {
	Titanium.Platform.openURL(url);
}

/**
 * Launch the email dialog and set email properties.
 */
function launchEmail() {
	var emailDialog = Titanium.UI.createEmailDialog();
	emailDialog.subject = "Hello from Titanium";
	emailDialog.toRecipients = ['foo@yahoo.com'];
	emailDialog.messageBody = '<b>Appcelerator Titanium Rocks!</b>';
	var f = Ti.Filesystem.getFile('cricket.wav');
	emailDialog.addAttachment(f);
	emailDialog.open();
}

/**
 * Use master window options set in Application properties and merge with passed in options.
 * @param options The options set for individual windows being loaded.
 */
function mixinWindowOptions(options) {
	_.extend(options, appProps.MASTER_WINDOW_PROPERTIES);
	return options;
}

function getScreenWidth() {
	return Ti.Platform.displayCaps.platformWidth;
}

function getScreenHeight() {
	return Ti.Platform.displayCaps.platformHeight;
}


function getMonthName(index) {
	return monthNames[index];
}

function getDayName(index) {
	return dayNames[index];
}

function stringifyDate(date) {
	var m = (date.getMonth()+1) + '';
	if(m.length == 1) m = '0' + m;
	var d = date.getDate() + '';
	if(d.length == 1) d = '0' + d;
	return date.getFullYear() + "-" + m + "-" + d;
}

function previousDay(date) {
	return new Date(date.setDate(date.getDate() - 1));
}

function nextDay(date) {
	return new Date(date.setDate(date.getDate() + 1));
}

function previousMonth(date) {
	return new Date(date.setMonth(date.getMonth() - 1));
}

function nextMonth(date) {
	return new Date(date.setMonth(date.getMonth() + 1));
}

function getTextualAttributeFromEvent(obj) {
	var text;
	if((obj.source.useTitle != null) && (!_.isUndefined(obj.source.useTitle))) {
		text = obj.source.useTitle;
	} else if((obj.source.title != null) && (!_.isUndefined(obj.source.title))) {
		text = obj.source.title;
	} else {
		text = obj.source.text;
	}
	return text;
}

function autoSizeTable(table, rows)
{
	var height = 0;
	for(var i= 0, l = rows.length; i < l; i++)
	{
		height += rows[i].height;
	}
	table.height = height;
}

function clickMapButtonMultiple(event)
{
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_event_map',
	    url: 'event_map.js',
		title: 'Multiple events',
		eventLocations:filteredEventLocations,
		linkToEvents:true,
		linkToNativeMaps:false,
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	}));
	loadWindow(newWin);	
}

function emptyString(v) {
	if(v == null || v.length == 0) {
		return true;
	} else {
		return false;
	}
}

function stringOrNull(v) {
	if(v == null || v.length == 0) {
		return null;
	} else {
		return v;
	}	
}

/*
 * Replaces the contents of the specified table with a row that says "Loading..."
 */
function showLoading(tableView)
{

	if(isIOS()) {

		Ti.API.trace('showLoading()');
	
		Ti.UI.actInd = Ti.UI.createActivityIndicator({
			style: Ti.UI.iPhone.ActivityIndicatorStyle.DARK,
			height: 60,
			width:100,
			message: 'Loading...',
			color: '#999',
			height:60,
			font: { fontSize: 15 },
			top: 0,
			bottom: 0,
		});
		
		var row = new TableViewRow({
			layout:'vertical',
			backgroundColor:'#fff',
			height: 60,
		});
		row.add(Ti.UI.actInd);
		tableView.setData([row]);

		Ti.UI.actInd.show();
		
		Ti.App.addEventListener('app:stopActivityIndicator', function(event) { 
			Ti.UI.actInd.hide();
		});
		
	}
	Ti.API.trace('showLoading() finished');
}

/*
 * Returns the inner height of the specified window.
 */
function getWindowHeight(window)
{
	// TODO: Ensure this works for iPad
	// TODO: Ensure this works for android
	var h = window.height;
	
	if(isIPad())
	{
		// Don't need to adjust height
	}
	else if(!window.fullscreen)
	{
		if(window.height == 416)
		{
			// Do nothing - 416 is the height of a window with a nav bar and settings bar shown
			
		}
		else if(!window.navBarHidden)
		{
			h -= 44;				// Navigation bar is 44 high
		}
	}
	return h;
}


function loadMonthSelectWindow(title)
{	
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_select_month',
	    url: 'select_month.js',
		title: title,
		showCategoryFilter: false,
		isCalendarMonth: true,
		barColor: win().barColor,
	}));

	loadWindow(newWin);
}

/*
 * Loads the events window for the specified month.
 * @param event the ID 
 */
function loadEventsWindowForMonth(event)
{
	// The row that triggers this event should have an id of "yyyymmdd"
	var d = event.source.id;
	setSelectedDate(new Date(d.substring(0,4), d.substring(4,6) - 1, d.substring(6,8)));
	
	loadEventsWindow(win().title, win().showCategoryFilter, win().isCalendarMonth);
}

/*
 * Loads the events list window and customises it according to the supplied options.
 * @param title The text to display in the navigation bar
 * @param showCategoryFilter True to show the "filter by category" bar
 * @param isCalendarMonth True to enable (and default to) showing a whole month of events
 */
function loadEventsWindow(title, showCategoryFilter, isCalendarMonth)
{
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_view_events_list',
	    url: 'view_events_list.js',
		layout: 'vertical',
		title: title,
		showCategoryFilter: showCategoryFilter,
		isCalendarMonth: isCalendarMonth,
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	}));
	loadWindow(newWin);
}

/* 
 * Returns the number of days in the specified month
 * @param iYear The year (e.g. 2011)
 * @param iMonth The month (0-11)
 */
function daysInMonth(iMonth, iYear)
{
	return 32 - new Date(iYear, iMonth, 32).getDate();
}

Ti.App.loadingWindowVisible = false;

function showLoadingWindow()
{
	if(isIOS()) {
		if(Ti.App.loadingWindowVisible)
		{
			Ti.API.warn('showLoadingWindow() called when the window is already visible');
			return;
		}
		else
		{
			Ti.API.debug('showLoadingWindow()');
		}
		
		var fullscreenWindow = new Window({
			backgroundColor: '#333',
			opacity: 0.80,
		});
		
		var loading = Ti.UI.createActivityIndicator({
			width: 90,
			height: 90,
		});
		
		if(isAndroid()) {
			loading.message = "You must have an\ninternet connection to\nuse Vic Events.";
		} else {
			loading.style = Ti.UI.iPhone.ActivityIndicatorStyle.BIG;
		}
		
		
		Ti.App.addEventListener('app:hideLoadingWindow', function(e) {
			Ti.App.loadingWindowVisible = false;
			fullscreenWindow.close();
			loading.hide();
		});	

		fullscreenWindow.add(loading);
		loading.show();
		
		fullscreenWindow.open();
		
		Ti.App.loadingWindowVisible = true;
	}
}

function hideLoadingWindow()
{
	Ti.App.fireEvent('app:hideLoadingWindow', {});
}

var tickTime = null;
var startTickTime = null;
function tick(msg) {
	if(msg == null) {
		msg = "";
	}
	
	if(tickTime == null) {
		tickTime = new Date().getTime();
		startTickTime = tickTime;
	}
	endTime = new Date().getTime();
	Ti.API.info(">> " + msg + " - " + (endTime - tickTime).toString());
	tickTime = endTime;
}

function finishingTick() {
	var endTime = new Date().getTime();
	Ti.API.info(">> Ticks took " + (startTickTime - endTime).toString() + " ms");
}

/*
 * Creates a link label with a standard colour that navigates to a URL when clicked
 * @param text The text to display on the link label
 * @param url The url to navigate to in the browser when the label is clicked
 * @param mixinOptions An object containing any width/height/etc. settings you wish to override
 */
function createLink(text, url, mixinOptions)
{
	var options = {
		text: text,
		font: { size: 10, },
		color: app.headerColor,
	};
	
	if(mixinOptions != null)
		_.extend(options, mixinOptions);
	
	var link = new Label(options); 
	link.addEventListener('click', function(){
		launchBrowser(url);
	});
	return link;
}

/*
 * Takes a date from the API (e.g. 'yyyy-mm-dd') and returns a
 * Javascript date object for that date.
 */
function parseDate(dateString)
{
	var bits = dateString.split('-');
	return new Date(parseInt(bits[0]), parseInt(bits[1])-1, parseInt(bits[2]));
}

/*
 * Joins all supplied non-empty strings with the supplied separator
 * @param separator The separator to use
 * @param strings An array of strings to join
 */
function joinStrings(separator, strings)
{
	var returnValue = '';

	for(var i =0; i < strings.length; i++)
	{
		if(!emptyString(strings[i]))
		{
			if(returnValue != '')
			    returnValue += separator;
			returnValue += strings[i];
		}
	}

	return returnValue;
}

/*
 * Returns a single string containing the location, street, city, state and postcode
 * of an event in a string formatted over two (or more) lines.
 */
function getEventLocation(event)
{
	var cityStatePostcode = joinStrings(', ', [ event.city, event.state, event.postcode ]);
	
	var fullLocation = joinStrings('\r\n', [event.location, event.street, cityStatePostcode]);
	
	return fullLocation;
}

// Android internet connectivity check. Use events on Activity to show hide.
if(isAndroid()) {
	var loading = null;
	
	Ti.Network.addEventListener("change", function(event){
		
		if(loading != null && Ti.Network.online) {
			Ti.API.debug(">>> ANDROID: Network change - hiding act ind.");
			loading.hide();
		} else {
			Ti.API.debug(">>> ANDROID: Network change - showing act ind");
			loading.show();
		}
	});
	
	Titanium.Android.currentActivity.addEventListener("resume", function(){
		Ti.API.debug(">>> ANDROID: RESUME.");
		if(loading != null && !Ti.Network.online) {
			loading.show();
		}
	});
	
	Titanium.Android.currentActivity.addEventListener("create", function(){
		Ti.API.debug(">>> ANDROID: Creating act indicator.")
		 	loading = Ti.UI.createActivityIndicator({
				width: 90,
				height: 90,
				message: 'You must have an\ninternet connection to\nuse Vic Events.'
			});
		
		if(loading != null && !Ti.Network.online) 
			loading.show();
	});

}
