function mapPin(id, pinTitle, lat, lng) {
	var pin = Titanium.Map.createAnnotation({
		latitude:lat,
		longitude:lng,
		title:pinTitle,
		pincolor:Titanium.Map.ANNOTATION_RED,
		animate:true,
		event_id:id
	});
	
	return pin;
}

function makeHasChildIndicator()
{
	return new View({
		width:8, 		// TODO: Native size of this image is 14 
		height:12, 		// TODO: Native size of this image is 21
		right: 10, 
		backgroundImage: imagePath('has-child.png'),
	});
}

/**
 * Make a row that is linked to something via a callback
 * Adding the id to every element allows every element to be handled by the click handler.
 * Android needs to have both left and right labels. A title on the row will get removed if a label is placed in the row.
 */
function makeLinkedStyledRow(rowId, left_label, right_label, unused_TODO_remove, callback, includeSeparator) {

	var linkRow = new TableViewRow({
		id:rowId, 
		text:left_label,
		backgroundColor:"#FFF",
		height: 'auto',
		className:'linkedRow',
		width: defaultRowWidth(),
		backgroundSelectedColor: '#999'
	});
	
	if(isIOS()) {
		linkRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY;
		linkRow.layout = null;	
	}
	
	var LEFT_LABEL_LEFT = 10;
	var rightLabelWidth = 60; // This should be wide enough to hold any right text, eg "1234" (event count)
	var rightLabelRight = (isIOS() ? 10 : 40);
	
	var leftLabel = new Label({
		id:rowId, 
		text:left_label, 
		top: 12,
		bottom: 12,
		left:LEFT_LABEL_LEFT, 
		width: 'auto',
		height: 'auto',
		backgroundColor:'transparent',
		color: app.headerColor,
		font: { fontSize: 13, fontWeight: 'bold', },
	});

	// Memory sink for Android
	if(isIOS()) {
		var img = makeHasChildIndicator();
		img.useTitle = left_label;
		linkRow.add(img);
	}
	
	linkRow.add(leftLabel);
	
	// MT: Resize to add padding on double-line rows
	if(leftLabel.width > 200)
		leftLabel.width = 200;

	if(!emptyString(right_label)) {
		var rightLabel = new Label({
			id:rowId,
			text:'(' + right_label + ')',
			useTitle: left_label, 
			textAlign:'right', 
			top: 11, // Not sure why but this has to be 1px less than the left label...
			height: 'auto',
			color: '#666', 
			width:rightLabelWidth, 
			backgroundColor:'transparent',
			font: { fontSize: 13,  },
		});
		//rightLabel.left = leftLabel.width + LEFT_LABEL_LEFT + 5;
		
		rightLabel.right = 30; 
		linkRow.add(rightLabel);	
	}
	
	if(includeSeparator)
		addFakeSeparator(linkRow);
	
	$(linkRow).click(function(event) {
		callback(event);
	});
	
	return linkRow;


/*
	var linkRow = new TableViewRow({
		id:rowId, 
		text:left_label,
		color:"#FFF",
		backgroundColor:'#FFF',
		height:40,
		className:'linkedRow',
		width: defaultRowWidth(),
	});
	
	if(section_header != null) {
		linkRow.header = section_header;
	}
	
	var rightLabelWidth = 60; // This should be wide enough to hold any right text, eg "1234" (event count)
	var rightLabelRight = (isIOS() ? 10 : 40);
	
	var img = new ImageView({
		backgroundImage: coloredImagePath('arw-right.png'),
		width: 30,
		height: 50,
		right: 0,
		useTitle: left_label,
	});
	
	linkRow.add(img);
	
	var leftLabel = new Label({
		id:rowId, 
		text:left_label, 
		left:10, 
		height:'auto', 
		width:'auto', 
		backgroundColor:'transparent',
		color: app.headerColor,
	});
	linkRow.add(leftLabel);
	
	// MT: Resize to add padding on double-line rows
	if(leftLabel.width > 200)
		leftLabel.width = 200;

	if(linkRow.height < leftLabel.height + 20)
		linkRow.height = leftLabel.height + 20;
	
	if(!emptyString(right_label)) {
		var rightLabel = new Label({
			id:rowId,
			text:' (' + right_label + ')',
			useTitle: left_label, 
			textAlign:'left', 
			height:'20', 
			top: 10,
			width:rightLabelWidth, 
			backgroundColor:'transparent'
		});
		rightLabel.left = leftLabel.width + 10;
		linkRow.add(rightLabel);	
	}
	
	$(linkRow).click(function(event) {
		callback(event);
	});
	
	return linkRow;
*/
}

var evtField = {};
evtField.DATES = 0;
evtField.LOCATION = 1;
evtField.TITLE = 2;
evtField.BLURB = 3;


evtField.FieldsWithDates = [ evtField.DATES, evtField.LOCATION, evtField.TITLE, evtField.BLURB ];
evtField.NormalFields = [ evtField.LOCATION, evtField.TITLE, evtField.BLURB ]

function displayDate(firstDateString, optionalSecondDateString)
{
	var bits = firstDateString.split('-');
	var returnValue = bits[2] + '/' + bits[1] + '/' + bits[0];
	
	// If a second date is supplied, and is different than the first date,
	// add it to the output string
	if(optionalSecondDateString != null && optionalSecondDateString != firstDateString)
	{
		bits = optionalSecondDateString.split('-');
		returnValue += ' - ' + bits[2] + '/' + bits[1] + '/' + bits[0];
	}
	return returnValue;
}

/*
 * Adds a fake separator to the top of a row
 */
function addFakeSeparator(row)
{
	if(isIOS()) {
		var fakeSeparator = new View({ height: 1, top:0, left:0, width: '100%', backgroundColor:'#e0e0e0'});
		row.add(fakeSeparator);
	}
}

function makeSummaryRow(evt, section_header, callback, fields) {

	var id = evt.id, title = evt.title, body = evt.body;
	var eventRow = new TableViewRow({
		id:id, 
		hasChild:false, 
		header:section_header, 
		className:'eventRow',
		backgroundColor:'#fff',
		backgroundSelectedColor: '#999'
	});

	if(isIOS()) {
		eventRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY;
	}
	
	var t = 10;
	
	var w = getTableContentWidth(win());

	for(var f in fields)
	{
		var newLabel = null;
		switch(fields[f])
		{
			case evtField.DATES:
				newLabel = new Label({
					id:id, 
					text: displayDate(evt.date.start, evt.date.end), 
					top:t,
					width:w,
					height:15,
					color:'#999',
					font: { fontSize: 11 },
					backgroundColor:'transparent',
				});
				break;
				
			case evtField.LOCATION:
				var place = evt.council.name;
				if(place != '')
				{
					newLabel = new Label({
						id:id, 
						text:place, 
						top:t,
						width:w,
						height:15,
						color:'#999',
						font: { fontSize: 11 },
						backgroundColor:'transparent',
					});
				}
				break;
				
			case evtField.TITLE:
				newLabel = new Label({
					id:id, 
					text:title, 
					ellipsize:true,
					top:t,
					backgroundColor:'transparent',
					height:22,
					width:w,
					color:app.headerColor,
					font: { fontSize: 16, fontWeight: 'bold', },
					textAlign:'left',
				});
				t += 3;
				break;
				
			case evtField.BLURB:

				
				var b = "";
				if(isIOS()) {
					b = body.replace(/\r\n/g,"\n").replace(/\n\n/g,"\n");
				} else {
					b = body.replace(/\r\n/g," ").replace(/\n\n/g," ");
				}
				newLabel = new Label({
					id:id, 
					text:b, // Remove space between paragraphs
					top:t,
					height:32 + (15 - 0),
					width:w,
					color:'#333',
					ellipsize:false,
					wordWrap: true,
					font: { fontSize: 13 },
					backgroundColor:'transparent',
					textAlign:'left',
				});
				break;
		}
		
		if(newLabel != null)
		{
			eventRow.add(newLabel);
			t += newLabel.height;
		}
	}

	eventRow.height = Math.max(80, t + 15);


	// MT: Possible for there to be no council associated with an event?
	// If so, shuffle up the remaining information
	var placeHeight = 15;
	if(place == '')
	{
		placeHeight = 0;
	}
	else
	{
		placeHeight = 15;
	}
		
		
	$(eventRow).click(function(event) {
		callback(event);
	});
	
	return eventRow;
}


function makeEventRow(id, title, body, place, section_header, callback) {

	var eventRow = new TableViewRow({
		id:id, 
		hasChild:false, 
		header:section_header, 
		height:80,
		className:'eventRow',
		backgroundColor:'#fff',
	});
	
	// MT: Possible for there to be no council associated with an event?
	// If so, shuffle up the remaining information
	var placeHeight = 15;
	if(place == '')
	{
		placeHeight = 0;
	}
	else
	{
		placeHeight = 15;
		var placeLabel = new Label({
			id:id, 
			text:place, 
			top:5,
		    left:10,
			height:placeHeight,
			color:'#999',
		 	font: { fontSize: 10 },
		 	backgroundColor:'transparent'
		});
		eventRow.add(placeLabel);
	}
		
	var titleLabel = new Label({
		id:id, 
		text:title, 
		ellipsize:true,
	    top:5 + placeHeight,
	    left:10,
		height:22,
		width:'auto',
		color:app.headerColor,
	 	font: { fontSize: 16 },
		textAlign:'left',
	});
	eventRow.add(titleLabel);
	
	var bodyLabel = new Label({
		id:id, 
		text:body.replace(/\r\n/g,"\n").replace(/\n\n/g,"\n"), // Remove space between paragraphs
	    top:25 + placeHeight,
		left:10,
		right:10,
    	height:30 + (15 - placeHeight),
    	width:'auto',
		color:'#000',
		ellipsize:true,
 		font: { fontSize: 11 },
	 	backgroundColor:'transparent',
	 	textAlign:'left',
	});
	eventRow.add(bodyLabel);
	
	$(eventRow).click(function(event) {
		callback(event);
	});
	
	return eventRow;
}

/*
 * Creates a standard error row to display the specified error message
 * @param error Either an error object returned from the API ( { code:x, message:x }) or a string.
 */
function makeErrorRow(error) {

	var errorRow = new TableViewRow({
		hasChild:false, 
		className:'errorRow',
		backgroundColor:'#fff',
		height: 'auto'
	});
	
	if(isIOS()) {
		errorRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
	}
	
	var width = getTableContentWidth(win());
	
	// Check if this error message should be overridden
	var msg;
	
	if(typeof error == 'string')
	{
		msg = error;
	}
	else
	{
		var msg = error.message;
		if(error.code != null)
		{
			msg = L('error_' + error.code, error.message);
		}
	}
	
	var minSize = new View({ width: 1, height: 60, top:0, left: 0, backgroundColor: 'transparent',});
	errorRow.add(minSize);
	
	var paddingContainer = new View({
		width:width,
		height:'auto',
		backgroundColor:'transparent',
	})
	
	var titleLabel = new Label({
		text: msg, 
		height:'auto',
		width:width,
		color:'#999',
		font: { fontSize: 15 },
		textAlign:'center',
 		backgroundColor: 'transparent',
 		top: 15,
 		bottom: 15,
	});
	
	paddingContainer.add(titleLabel);
	errorRow.add(paddingContainer);

	return errorRow;
}

var app = {
	headerColor: '#009dc9',
	backgroundBlueColor: '#d5eaef',
	tablePadLeft: 15,
	textColor: '#333',
	tipTextColor: '#666', 
};

function makeEventTitleRow(table, rowTitle) {
	var rowHeight = 40;
	
	var rowTitleLabel = new Label({
		text:rowTitle, 
		top: 10,
		bottom: 8,
		left: 13,
		width: getTableContentWidth(win()), 
		height:'auto',
		color: app.headerColor,
 		font: { fontSize: 16, fontWeight: 'bold', },
 		backgroundColor: 'transparent',
	});

	var eventTitleRow = new TableViewRow({
		height:'auto',
		width: table.width, 
		backgroundColor: '#fff'
	});
	
	if(isIOS()) {
		eventTitleRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
	}
	
	eventTitleRow.add(rowTitleLabel);

	return eventTitleRow;		
}

function makeViewAllEventsRow() {
	var rowHeight = 'auto';
	
	var rowTitleLabel = new Label({
		text:'View all...', 
		left:10,
		right:10, 
		top: 10,
		bottom: 10,
		height:'auto',
		color: app.headerColor,
 		font: { fontSize: 12, fontWeight: 'bold', },
 		textAlign:'center',
 		backgroundColor: 'transparent',
	});

	var eventTitleRow = new TableViewRow({
		height:'auto', 
		backgroundColor: app.backgroundBlueColor,
	});

	addFakeSeparator(eventTitleRow);
	
	eventTitleRow.add(rowTitleLabel);

	return eventTitleRow;		
}

function makeEventMapRow(table, callback) {
	var rowHeight = 35;
	
	var eventMapRow = new TableViewRow({
		height:rowHeight,
		width: table.width, 
		backgroundColor: 'white'
	});
	
	var mapButton = Ti.UI.createButton({
		backgroundImage:coloredImagePath('btn_map.png'),
		height:40, 
		width:89
	});
	
	eventMapRow.add(mapButton);
	
	$(mapButton).click(function(event){
		callback(event);
	});
	
	return eventMapRow;
}

function makeEventDetailRow(table, rowTitle, rowValue, callback) {
	
	var rowHeight = 35;
	
	var rowTitleLabel = new Label({
		text:rowTitle,
		left:13, 
		top:10,
		width:80, 
		textAlign:'left',
		font: { fontSize: 13, fontWeight: 'bold' },
		color: app.textColor,
		backgroundColor: 'transparent',
	});
	
	if(isAndroid()) {
		rowTitleLabel.height = 'auto';
	} else {
		rowTitleLabel.height = 15;	
	}
	
	var rowValueLabel = new Label({
		text:rowValue, 
		right:15,
		width: getTableContentWidth(win()) - 90,
		top: 11,
		bottom: 10,
		height:'auto',
		textAlign:'left',
		backgroundColor: 'transparent',
		font: { fontSize: 13 },
		color: app.textColor,
	});
	
	var eventDetailRow = new TableViewRow({
		height:'auto',
		width: table.width, 
		backgroundColor: 'white'
	});

	if(isIOS()) {
		eventDetailRow.layout = null;
		eventDetailRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
	}

	eventDetailRow.add(rowValueLabel);
	eventDetailRow.add(rowTitleLabel);
	
	if(callback != null) {
		
		rowValueLabel.color = app.headerColor; 
		
		if(isIOS()) {
			eventDetailRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY;
		} else {
			eventDetailRow.backgroundSelectedColor = '#999';
		}
		
		$(rowTitleLabel).click(function(event) {
			callback(event);
		});
		$(rowValueLabel).click(function(event) {
			callback(event);
		});		
	}

	addFakeSeparator(eventDetailRow);

	return eventDetailRow;
}

/*
 * @param isEnabled True if this should show as enabled, and fire the callback when it is clicked. False to show as disabled and to show an error message when clicked.
 * @param isMultipleEvents True if this is from a listing of multiple events, false if it is from a single event. Only controls what error message is shown if isEnabled is false.
 * @param callback The function to call if this is enabled and the user clicks it.
 */
function setNavBarMap(isEnabled, isMultipleEvents, callback)
{
	// Create a container View so that we can add a bit of padding
	// to the left and right of the image (for appearance and also so
	// it has a larger hit area)
	var customRightNavButton = new View({
		width:30,
		right:10,
	});
	
	var imageView = new View({
		backgroundImage:imagePath(isEnabled ? 'map_marker.png' : 'map_marker_off.png'),
		height:22,
		width:17,
		left:5,
		touchEnabled:false,
	})
	
	customRightNavButton.add(imageView);
	win().rightNavButton = customRightNavButton;

	if(isEnabled) 
	{
		customRightNavButton.addEventListener('click', function(event){
			callback(event);
		});
	}
	else
	{
		// This is disabled - show a message 
		var message;
		if(isMultipleEvents)
			message = L('no_map_data_multiple');
		else
			message = L('no_map_data_single');

		customRightNavButton.addEventListener('click', function(event){
			messageBox(message);
		});
	}	
}


function makeAndroidMapFooter(isEnabled, tableView, callback) {
	
	Ti.API.info(">>> There are coords for events " + isEnabled);
	
	if (!isEnabled) {
		return;
	}
	
	var mapFooter = Ti.UI.createView({height:70,width:getScreenWidth(),backgroundColor:'white'});
	
	var mapButton = Ti.UI.createButton({
		top: 15,
		height:40, 
		width:200,
		backgroundImage:coloredImagePath('btn_map_long.png')
	});
	mapFooter.add(mapButton);
	
	$(mapButton).click(function(event){
		callback(event);
	});
	
	tableView.footerView.add(mapFooter);
	tableView.footerView.visible = true;
}


function setNavBarList(callback) {
	
	// Create a container View so that we can add a bit of padding
	// to the left and right of the image (for appearance and also so
	// it has a larger hit area)
	var customRightNavButton = new View({
		width:30,
		right:10,
	});
	
	var imageView = new View({
		backgroundImage:imagePath('list.png'),
		height:16,
		width:18,
		left:5,
		touchEnabled:false,
	})
	
	customRightNavButton.add(imageView);

	win().rightNavButton = customRightNavButton;
	
	$(customRightNavButton).click(function(event){
		callback(event);
	});
	
}

function createFilterHeader(text)
{
	var screenTitleLabel = new Label({
		text: text, 
		textAlign:'center',
		backgroundImage: coloredImagePath('filter-bar.png'),
		color: '#fff',
		font: { fontSize: 12, fontWeight:'bold' },
		top:0, 
		height:30,
	});
	
	if(isAndroid()) {
		screenTitleLabel.width = getScreenWidth();
	}
	
	return screenTitleLabel;
}

/**
 * Creates the calendar header control
 * @param filter
 * @param includeInstructions True to include the "Tap to select..." message
 * @param supportsMonthOnly True if the list of dates this control creates when tapped will offer the "all month" option
 * @param isMonthOnly True if this control is currently in month-only mode (e.g. display "July 2011" rather than "1st July 2011")
 * @param getFunction Optional parameter used to retrieve the selected date. If not set, this will call getSelectedDate();
 * @param setFunction Optional parameter used to set the selected date. If not set, this will call setSelectedDate();
 */
function calendarHeaderView(filter, includeInstructions, supportsMonthOnly, isMonthOnly, getFunction, setFunction) {

	// If you don't supply a get/set function, work on the default selected
	// date variable
	if(getFunction == null)
		getFunction = getSelectedDate;
	if(setFunction == null)
		setFunction = setSelectedDate;
	

	var containerView = new View({
		height:50,
		width:'100%',
	});

	var calendarView = new View({
		height:50,
		backgroundColor:getSharedPropertyAsString(appProps.KEY_FILTER_DARK_BAR_COLOR) || '#bbb',
		top:0
	});
		
	if(filter != null) {
		var screenTitleLabel = createFilterHeader(filter.name);
		containerView.add(screenTitleLabel);
		containerView.height = containerView.height + screenTitleLabel.height;
		calendarView.top = 30;
	}
	
	// Centering Date.
	var centerView = Ti.UI.createView({
		width: win().width - 100,
		height:45,
		top:0
	});
	
	var dateText = "";
	var date = getFunction();
	
	var dateLabel = new Label({
		top:5,
		height:20,
		backgroundColor:'transparent',
		color:'#FFF',
		font: { fontSize: 15, fontWeight:'bold' },
		textAlign:'center',
		text:dateText
	});
	
	if(!includeInstructions) {
		if(isIOS())
			dateLabel.height = containerView.height;
		else
			dateLabel.height = 50;
	}
	
	centerView.add(dateLabel);
	
	if(includeInstructions) {
		var instructionsLabel = new Label({
			className:'calendar_view_instructions',
			text:isMonthOnly ? L('tap_to_change_month_text') : L('tap_to_change_text'),
			color: '#fff',
			font: { fontSize: 12, fontWeight: 'bold', },
			width: 'auto',
			textAlign:'center'
		});
		
		if(isAndroid()) {
			instructionsLabel.top = dateLabel.height + 5;
		}
		else
		{
			instructionsLabel.top = dateLabel.top + dateLabel.height;
		}
		
		
		centerView.add(instructionsLabel);
	} else {
		dateLabel.top = null;
	}
	calendarView.add(centerView);
	
	// Left button
	var leftView = Ti.UI.createView({
		width:50,
		height:50,
		top:0,
		left:0,
	});
	leftView.add(Ti.UI.createView({ width:14, height:21, backgroundImage: coloredImagePath('arrow-left-plain.png') }));
	calendarView.add(leftView);
	
	// Right button
	var rightView = Ti.UI.createView({
		width:50,
		height:50,
		top:0,
		right:0,
	});	
	rightView.add(Ti.UI.createView({ width:14, height:21, backgroundImage: coloredImagePath('arrow-right-plain.png') }));
	calendarView.add(rightView);
	
	containerView.add(calendarView);
	
	// Event handling
	if(includeInstructions)
	{
		$(centerView).click(function(event){
			var newWin = new Window(mixinWindowOptions({
			    id: 'win_date_picker',
			    url: 'filter_by_date.js',
				title:'Select a date',
				barColor: win().barColor,
				includeAllMonth: supportsMonthOnly,
			}));
	
			loadWindow(newWin);
		});
	}
		
	$(leftView).click(function(event){
		if (isMonthOnly) {
			setFunction(previousMonth(getFunction()));
		} else {
			setFunction(previousDay(getFunction()));
		}
		Ti.API.info(">>> You clicked the left");
		refreshTableView();
	});	
	$(rightView).click(function(event){
		if (isMonthOnly) {
			setFunction(nextMonth(getFunction()));
		} else {
			setFunction(nextDay(getFunction()));
		}
		refreshTableView();
	});
	
	
	// Fire this event when you want to update something in this view. 
	// Fire it like this - Ti.App.fireEvent('updateCalendarHeader');
	Ti.App.addEventListener('updateCalendarHeader', function(e)
	{
		Ti.API.info(">>> updateCalendarHeader - Updating calendar header");
		date = getFunction();
		
		Ti.API.info(">>> Got a date of " + date.toString());
		
		if(isMonthOnly) {
			Ti.API.info(">>> Updating calendar header in isMonthOnly mode - setting date to " + getMonthName(date.getMonth()) + ", " + date.getFullYear());
			dateLabel.text = getMonthName(date.getMonth()) + ", " + date.getFullYear();
		} else {
			Ti.API.info(">>> Updating calendar header in NOT isMonthOnly mode - setting date to " + date.getDate() + " " + getMonthName(date.getMonth()) + ", " + getDayName(date.getDay()));
			dateLabel.text = getDayName(date.getDay()) + ", " + date.getDate() + " " + getMonthName(date.getMonth()) + " " + date.getFullYear();
		}
	});
		
	return containerView;
}

/**
 *
 */
function makeCategoryFilterRow(callback) {
	
	var filterRow = new TableViewRow({
		id:'filter',
		backgroundColor:'transparent',
		height:30,
		hasChild:false,	
	});

	var defaultLabel = new Label({
		text:L('category_filter_row_text'),
		backgroundColor:'transparent',
		left:32,
		color:'#fff',
		font: { fontSize: 13 }
	});
	
	var imageView = new View({
		width:25,
		height:25,
		top:3,
		left:5,
		backgroundColor:'transparent',
		backgroundImage:coloredImagePath('arwlink.png'),
	})
	filterRow.add(imageView);
	filterRow.add(defaultLabel);
	
	$(filterRow).click(function(event) {
		callback(event);
	});
	
	return filterRow;
}

/**
 *
 */
function makeRemovableLabel(category, callback, removable) {
	
	if(category == null) {
		return null;
	}
	
	var maskContainer = new View({height:20,width:300});
	
	var container = new View({
		height:20,
		width:'auto',
		backgroundColor:'#FFF',
		borderColor:'#999',
		borderWidth:1,
		borderRadius:5,
		right:5,
		touchEnabled:true,
	});
	
	var tagLabel = new Label({
		id:category.id,
		text:category.name,
		textAlign:'left',
		left:7,
		top:2,
		bottom:2,
		right:20,
		font: { fontSize: 11 },
		height:20,
		width:'auto'
	});
	container.add(tagLabel);
	
	if(removable) {
		var removeButton = new Button({
			backgroundImage:'../images/close.png',
			height:7,
			width:7,
			top:2,
			bottom:2,
			right:7,
			stop:true
		});

		container.add(removeButton);

		$(maskContainer).click(function(event) { callback(event); });

		$(removeButton).click(function(event){
			callback(event);
		});

	}
	
	maskContainer.add(container);
	
	return maskContainer;
}

/**
 *
 */

function addCategoryFilter(event)
{
	doAddCategoryFilter(event, true)
}

function addCategoryFilterAndWriteFilter(event)
{
	doAddCategoryFilter(event, false)
}

function doAddCategoryFilter(event, returnOnSelect) {
	
	if(event.source.stop == true) {
		return;
	}
	
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_categories',
	    url: 'filter_by_category.js',
		title:'Categories',
		returnOnSelect:returnOnSelect,
		barColor: win().barColor,
		selectedDate: getSelectedDate(),
	}));
	
	loadWindow(newWin);
}

/**
 *
 */
function removeCategoryFilter(event) {
	removeSelectedCategory();	
	selectedCategoryLabel.hide();
}

function viewEventWithId(id, untrackable, hideMapControl) {
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_event_details',
	    url: 'event.js',
		title:'Event',
		eventId:id,
		barColor:win().barColor,
	}));
	
	if(untrackable) {
		newWin.untrackable = true;
	}
	
	if(hideMapControl) {
		newWin.hideMapControl = hideMapControl;
	}
	
	loadWindow(newWin);	
}

function defaultRowWidth()
{
	return getStandardTableWidth() - 30;
}

/*
 * Creates a table that fills the whole screen, and corrects the top padding
 * so that it does not appear to be twice the padding as appears on the sides.
 * @param top An optional integer which will be be used as the top position of the table, e.g. to account for other controls that appear above the table.
 */
function makeFullscreenTable(top)
{
	var tableHeight = isAndroid() ? (getScreenHeight() - 50) : getWindowHeight(win());
	
	if(top != null) {
		tableHeight -= top;
	}
	
	// Using this header view removes excess padding from the top of the table on iOS
	var headerView = new View({width:100, height: 1, backgroundColor:'transparent',});
	
	var tableOptions = {
		backgroundColor:'transparent',
		width: getStandardTableWidth(), 
		separatorColor: 'transparent',
		headerView: headerView,
		height: tableHeight,
	};
	
	if(isIOS()) {
		tableOptions.separatorStyle = Ti.UI.iPhone.TableViewSeparatorStyle.NONE; // iPad: Removes line (or "chin") from bottom of the table
		tableOptions.style = Ti.UI.iPhone.TableViewStyle.GROUPED;		
	} else {
		//tableOptions.backgroundColor = '#e0e0e0';
		tableOptions.separatorColor = '#e0e0e0';
		//tableOptions.borderColor = '#e0e0e0';
		//tableOptions.borderWidth = 1;	
	}

	if(top != null)
	{
		// If the window has a vertical layout, it will automatically appear below the
		// last added element. Otherwise, specifically set the top property.
		if(win().layout != 'vertical') {
			tableOptions.top = top;
		}
	}
	
	return new TableView(tableOptions);
}


/*
 * Creates the category filter object. The view itself can be accessed using
 * object.filterView - and the view has an object.update() method which can
 * be used to update the text and/or remove filter link.
 */
function makeCategoryFilter()
{
	var filterHeading = {};
	
	filterHeading.filterView = new View({
		backgroundColor:'transparent',
		width:'94%',
		height:35,
		borderWidth:0,
	});
	
	filterHeading.filterText = new Label({
		height:25,
		left:32,
		text:L('category_filter_row_text'),
		backgroundColor:'transparent',
		color: 'white',
		width: 'auto',
		font: { fontSize: '13.5', fontWeight: 'bold', },
	})
	
	filterHeading.filterImage = new View({
		backgroundImage: coloredImagePath('arwlink.png'),
		width:25,
		height:25,
		left: 5,
	})
	
	filterHeading.removeLink = new Label({
		text: '(remove filter)',
		id: 'close_view',
		width: 'auto',
		height: 25,
		width: 100,
		right: 10,
		backgroundColor: 'transparent',
		// TODO: small font, hyperlink
		color: 'white',
		textAlign: 'right',
		font: { fontSize: '12.5' },
	});
	
	if(isAndroid()) {
		filterHeading.removeLink.height = 50;
		filterHeading.filterView.height = 50;
	}	
	
	filterHeading.filterView.add(filterHeading.filterText);
	filterHeading.filterView.add(filterHeading.filterImage);
	filterHeading.filterView.add(filterHeading.removeLink);
	
	filterHeading.filterView.addEventListener('click', function(event) {
		if(event.source.id == 'close_view')
		{
			removeSelectedCategory();
			clearPrimaryFilter();
			refreshTableView();
		}
		else
		{
			showCategorySubFilter(event);
		}
	});
	
	filterHeading.update = function() {
		var selectedCategory = getSelectedCategory();
		//var selectedCategory = getPrimaryFilter();
		Ti.API.info(">>>> Updating the filter heading...");
		
		if(getPrimaryFilter() != null ) {
			Ti.API.info(">>> Primary filter has something in it " + getPrimaryFilter().name);
		} else {
			Ti.API.info(">>> Primary filter has nothing in it either");
		}
		
		if(selectedCategory != null)
		{
			Ti.API.info(">>>> Updating the filter heading with " + selectedCategory.name);
			// TODO
			filterHeading.filterText.text = selectedCategory.name;
			filterHeading.filterText.width = 'auto';
			filterHeading.removeLink.show();
		}
		else
		{
			Ti.API.info(">>>> Updating filter header but no filter is selected!!!");
			filterHeading.filterText.text = L('category_filter_row_text');
			filterHeading.removeLink.hide();
		}
	}
	
	return filterHeading;
}

function makeNoResultsRow(title)
{
	return new TableViewRow({ 
		title:title,
		backgroundColor: 'white',
		height: 60,
	});
}

