Ti.include('../redux.js');
Ti.include('../includes/sharing.js');

var window = win();

var characterLimit = window.characterLimit;
var currentEvent = window.currentEvent;
var postCallback = window.postCallback;
var oAuthAdapter = window.oAuthAdapter;
var currentUsername = window.currentUsername;
var logoutCallback = window.logoutCallback;
var shareTemplateName = window.shareTemplateName;

var contentWidth = (isAndroid() ? getScreenWidth() :  window.width) - 50;

var buttons = [];
var btnDone = null;

if(isIOS())
{
	var flexSpace = Ti.UI.createButton({ systemButton: Ti.UI.iPhone.SystemButton.FLEXIBLE_SPACE, })
	btnDone = Ti.UI.createButton({ systemButton: Ti.UI.iPhone.SystemButton.DONE, });
	buttons = [ flexSpace, btnDone ];
}
else if(isAndroid()){
	// Android requires the width is specified
	window.width = Ti.Platform.displayCaps.platformWidth;
	window.orientationModes = [ Ti.UI.PORTRAIT ];
}

var closeArea = null;
if(isIOS())
{
	var headingContainer = new View({ width:'100%', height: 40, backgroundColor: '#eee', });
	var heading = new Label({ text: win().title, left: 25, top: 6, backgroundColor: 'transparent', font: { fontWeight: 'bold', }, color: app.textColor, });
	closeArea = new View({ width: 35, height: 35, top: 0, right: 0, });
	var closeImage = new View({ left:5, bottom: 5, width: 15, height: 15, backgroundImage: imagePath('close-gray.png'), });
	closeArea.add(closeImage);
	headingContainer.add(heading);
	headingContainer.add(closeArea);
	window.add(headingContainer);
}

var label = new Label({
	width: contentWidth,
	height: 30,
	top: 5,
	text: 'Say something about this event:',
	font: { fontSize: 14, },
	backgroundColor: 'transparent',
	color: app.textColor,
})

window.add(label);


var shareMessage;

if(shareTemplateName == 'twitter')
{
	shareMessage = "I'm going to check this out: "
	var len = shareMessage.length + (currentEvent.url != null ? currentEvent.url.length : 0) + 3;
	if(currentEvent != null)
	{
		if(currentEvent.title != null)
		{
			var availableLength = characterLimit - len - 3;
			if(availableLength > 4 && currentEvent.title.length > availableLength)
			{
				shareMessage += currentEvent.title.substring(0, availableLength - 3) + "... ";
			}
			else
			{
				shareMessage += currentEvent.title + ' - ';
			}
		}
		
		if(currentEvent.url) shareMessage += currentEvent.url;
	}
}
else
{
	shareMessage = "I'm going to check this out...";
}





var textarea = Titanium.UI.createTextArea({
	editable: true,
	value:shareMessage,
	height:65,
	width:contentWidth,
	top:5,
	font:{fontSize:14, },
	color:app.textColor,
	textAlign:'left',
	//appearance:Titanium.UI.KEYBOARD_APPEARANCE_ALERT,	
	//keyboardType:Titanium.UI.KEYBOARD_NUMBERS_PUNCTUATION,
	keyboardToolbar: buttons,
	borderWidth:1,
	borderColor:'#bbb',
	borderRadius:5,
	suppressReturn:false
	
});
window.add(textarea);


// Set up character limiting, if a characterLimit greater than zero was specified
// when the window was created
var winCharacterLimit = win().characterLimit;
if(winCharacterLimit != null && winCharacterLimit > 0)
{
	var limitLabel = new Label({
		width:contentWidth,
		height:20,
		top: 5,
		font: { fontSize: 14, },
		backgroundColor: 'transparent',
		text: textarea.value.length + ' of ' + winCharacterLimit + ' characters used',
	});
	
	window.add(limitLabel);
	
	// Track the last value of the textarea so if the user tries to enter more
	// characters than are allowed, we can reset it to the previous value
	var lastText = textarea.value;
	
	textarea.addEventListener('change', function(e){
		Ti.API.trace('change fired ' + textarea.value.length + ' of ' + winCharacterLimit + ' characters');
		if(textarea.value.length > winCharacterLimit)
		{
			textarea.value = lastText;
		}
		else
		{
			lastText = textarea.value;
		}
		limitLabel.text = textarea.value.length + ' of ' + winCharacterLimit + ' characters used';
	});
	
}

// create a button to close window
var buttonContainer = new View({
	width: contentWidth,
	height: 170,
	top: 15,
	backgroundColor:'transparent',
});

var closeButton = Titanium.UI.createButton({
	title:'cancel',
	height:40,
	width:115,
	right: 0,
	top: 0,
	backgroundImage: imagePath('turquoise/btn-dark.png'),
	font: { fontSize: 15, fontWeight: 'bold', },
});

var postButton = Titanium.UI.createButton({
	title:'share',
	height:40,
	width:115,
	left: 0,
	top: 0,
	backgroundImage: coloredImagePath('btn.png'),
	font: { fontSize: 15, fontWeight: 'bold', },
});

buttonContainer.add(postButton);
buttonContainer.add(closeButton);

window.add(buttonContainer);


var signOutText = "Tap here to log out";
if(currentUsername != null)
    signOutText = "Signed in as " + currentUsername + ".\r\nTap here to log out";

var logoutLabel = new Label({
	text:signOutText,
	color: app.headerColor,
	width:contentWidth,
	height: 40,
	top: 50,
	left: 0,
	font: { fontSize: 12, fontWeight: 'bold', },
	backgroundColor:'transparent',
});
buttonContainer.add(logoutLabel);


if(shareTemplateName == 'facebook')
{
	var notification = new Label({
		text:'The title, description and link to this event will be included in your Facebook wall post',
		color: app.textColor,
		width: contentWidth,
		height: 60,
		left: 0,
		top: 45,
		backgroundColor:'transparent',
		font: { fontSize: 14, },
	});
	
	logoutLabel.top += 60;
	buttonContainer.add(notification);
}

logoutLabel.addEventListener('click', function()
{
	logoutCallback();
	closeThisWindow();
});

var loadingIndicator = null;


if(isIOS())
{
	loadingIndicator = Ti.UI.createActivityIndicator({
		style: Ti.UI.iPhone.ActivityIndicatorStyle.DARK,
		width: 150,
		height: 60,
	});
	
	window.add(loadingIndicator);

	loadingIndicator.color = '#333';
}

function disableForm()
{
	postButton.enabled = false;
	textarea.enabled = false;
	textarea.editable = false;

	if(loadingIndicator != null)
		loadingIndicator.show();
}

function enableForm()
{
	postButton.enabled = true;
	textarea.enabled = true;
	textarea.editable = true;

	if(loadingIndicator != null)
		loadingIndicator.hide();
}

function doShare()
{
	textarea.blur();
	
	if(emptyString(textarea.value))
	{
		messageBox('Please enter a message to share.');
	}
	else
	{
		disableForm();
	
		postCallback(currentEvent, textarea.value, afterPostCallback);
	}
}

function afterPostCallback(success, optionalErrorMessage)
{
	if(!success)
	{
		enableForm();
		if(optionalErrorMessage)
		{
			messageBox('Sorry your message could not be posted at this time.\r\n\r\n' + optionalErrorMessage);
		}
		else
		{
			messageBox('Sorry, your message could not be posted at this time. Please try again later.');
		}
	}
	else
	{
		closeThisWindow();
	}
}

function closeThisWindow()
{
	textarea.blur();
	
	hideLoadingWindow();
	
	var t3 = Titanium.UI.create2DMatrix();
	t3 = t3.scale(0);
	window.close({transform:t3,duration:300});
}

postButton.addEventListener('click', doShare);

if(closeArea != null)
{
	closeArea.addEventListener('click', closeThisWindow);
}
closeButton.addEventListener('click', closeThisWindow);

if(btnDone != null)
{
	btnDone.addEventListener('click', function() { textarea.blur(); });
}