Ti.include('../redux.js');
Ti.include('../includes/sharing.js');

var currentEvent = {};
var awaitingFacebookPost = false;

function createShareView(text, image, callback)
{
	var sectionView = new View({
		height: 80,
		width: 76,
		backgroundColor:'transparent',
		top: 30,
	})
	
	sectionView.add(new View({
		width: 40,
		height: 30,
		backgroundImage: image,
		top: 10,
	}));
	
	sectionView.add(new Label({
		width: 75,
		top: 45,
		textAlign:'center',
		text: text,
		color:'white',
		font: { fontSize: 12, },
		height: 'auto',
		backgroundColor:'transparent',
	}));
	
	sectionView.addEventListener('click', callback);
	
	return sectionView;
}

function launchNativeMap(event) {
	var eventLat = currentEvent.coordinates.lat;
	var eventLng = currentEvent.coordinates.lng;
	
	launchBrowser('http://maps.google.com/maps?t=m&q=' + Ti.Network.encodeURIComponent(currentEvent.title) + '@' + eventLat + ',' + eventLng);
}

function clickEventMapButton(event)
{	
	if(handleShakeEvent)
		Ti.Gesture.removeEventListener('shake', showNewRandomEvent);

	var eventLat = currentEvent.coordinates.lat;
	var eventLng = currentEvent.coordinates.lng;

	if(eventLat == null || eventLat == 0) {
		messageBox('No lat/lng supplied for this event.');
		return;
	}
	
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_event_map',
	    url: 'event_map.js',
		title:'Event',
		linkToEvents: false,
		linkToNativeMaps: true,
		eventLocations :[ { lat:eventLat, lng:eventLng, title: currentEvent.title, subtitle: getEventLocation(currentEvent) }],
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	}));
	
	loadWindow(newWin, false);
}

function clickWebLink(event) {
	if(isAndroid()) {
		disableInteractivity();
	}
	launchBrowser(currentEvent.web);
}

function clickPhoneLink(event) {
	if(isAndroid()) {
		disableInteractivity();
	}
		
	var dialog = Ti.UI.createAlertDialog({
		title: null,
		message: currentEvent.phone,
		buttonNames: [ 'Cancel', 'Call'], 
	})
	
	dialog.addEventListener('click', function(e){
		// If the user clicks 'Call', open the telephone link
		if(e.index == 1)
		{
			launchBrowser('tel:' + currentEvent.phone.replace(/ /g,''));
		}
	});

	dialog.show();
}

function clickEmailLink(event) {
	if(isAndroid()) {
		disableInteractivity();
	}
	
	setShakeEventsEnabled(false, 'email dialog shown');
		
	var dialog = Ti.UI.createEmailDialog({
		toRecipients: [currentEvent.email],
	});
	
	dialog.addEventListener('complete', function() { setShakeEventsEnabled(true, 'email dialog complete'); });
	
	dialog.open();
}

var tableStartHeight = 0;

if(win().showShakerBar) {
	
	// Create the "Shake for another event" bar
	var IMAGE_BLOCK_SIZE = 35;
	var shakerView = new View({	
		top:0,
		height:50,
	    backgroundColor: getSharedPropertyAsString(appProps.KEY_FILTER_DARK_BAR_COLOR) || '#AAA',
	});
	var shakerLabel = new Label({
		text: (isIPad() ? 'Shake or tap here for another random event' : 'Shake or tap for another event'),
	    color:getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	    width: (isIPad() ? 350 : (isIPhone() ? 'auto' : 200)),
	    textAlign:'center',
	    backgroundColor: 'transparent',
	});
	shakerLabel.left = IMAGE_BLOCK_SIZE + (( (isIOS() ? win().width : getScreenWidth()) - shakerLabel.width - IMAGE_BLOCK_SIZE) / 2);
	var shakerImage = new View({
		width:27,
		height:19,
		backgroundImage:coloredImagePath('shake.png'),
		left: shakerLabel.left - IMAGE_BLOCK_SIZE,
	});
	shakerView.add(shakerImage);
	shakerView.add(shakerLabel);

	win().add(shakerView);

	shakerView.addEventListener('click', function() { showNewRandomEvent(); })
	Ti.Gesture.addEventListener('shake', showNewRandomEvent);

	tableStartHeight += shakerView.height;
}

var handleShakeEvent = win().showShakerBar;

function showNewRandomEvent()
{
	if(shakeEventsEnabled === false)
	{
		Ti.API.debug('showNewRandomEvent: ignoring because shakeEventsEnabled === false');
	}
	else
	{
		Ti.API.debug('showNewRandomEvent: loading new random event');
		disableInteractivity();
		loadRemoteEvent('random');
	}
}


// Images view
var scrollView = Titanium.UI.createScrollView({
	width: (isIPad() ? getStandardTableWidth() - (4 * app.tablePadLeft) : Titanium.Platform.displayCaps.platformWidth),
	top:10,
	height: 150,
	contentHeight: 150,
	layout:'horizontal',
	borderWidth: 0,
});
var tableViewHeader = new View({width:getScreenWidth(), height: 1, backgroundColor:'transparent',});

var w = win();

var tableView = new TableView({
	top: tableStartHeight,
	backgroundColor:'transparent',
	headerView: tableViewHeader, // to remove top padding
	footerView: (isIOS() ? null : makeShareFooter()),
	width: getStandardTableWidth()
});

if(isAndroid()) {
	tableView.separatorColor = '#FFF';
	tableView.borderColor = '#FFF';
	tableView.borderWidth = 0;
} else {
	tableView.separatorColor = 'transparent';
}

if(isIOS()) {
	tableView.style = Titanium.UI.iPhone.TableViewStyle.GROUPED;
	tableView.separatorStyle = Ti.UI.iPhone.TableViewSeparatorStyle.SINGLE_LINE;
}

function makeShareFooter() {
	
	var tableFooterView = new View({
		width: isAndroid() ? (getScreenWidth()) : getTableContentWidth(win()),
		height: 60,
		top: 10,
		backgroundColor:(isIOS() ? 'transparent' : 'white'),
		visible:(isIOS() ? true : false)
	});
	
	var buttonContainer = new View({
		width: getTableContentWidth(win()),
		height: 60,
		top: 10,
		backgroundColor:(isIOS() ? 'transparent' : 'white'),
	});
	
	if(isAndroid()) {
		buttonContainer.width = getStandardTableWidth();
	}
	
	var shareButton = new Button({
		width: 87,
		height: 40,
		left: 0,
		top: 0,
		font: { fontSize: 15, fontWeight: 'bold', },
	});
	
	if(isIOS()) {
		shareButton.title = ' share';
		shareButton.backgroundImage = coloredImagePath('btn.png');
		shareButton.image = imagePath('icon_share.png');
	} else {
		shareButton.image = coloredImagePath('btn_share_new.png');
		shareButton.width = 120;
		shareButton.height = 40;
	}
	
	var calendarButton = new Button({
		width: 161,
		top: 0,
		height: 40,
		font: { fontSize: 15, fontWeight: 'bold', },
	});
	
	if(isIOS()) {
		calendarButton.title = ' add to calendar';
		calendarButton.backgroundImage = coloredImagePath('btn.png');		
		calendarButton.image = imagePath('icon_calendar.png');
		calendarButton.left = 100;
		tableFooterView.add(shareButton);
		tableFooterView.add(calendarButton);
	}
	else
	{
		calendarButton.image = coloredImagePath('btn_add_calendar.png');	
		calendarButton.width = 150;
		calendarButton.height = 40;
		calendarButton.right = 0;
		
		var buttonContainer = new View({
			width: getTableContentWidth(win()),
			height: 60,
			top: 10,
			backgroundColor: 'white',
		});
		
		tableFooterView.add(buttonContainer);
			
		buttonContainer.add(shareButton);
		buttonContainer.add(calendarButton);
	}
	
	calendarButton.addEventListener('click', function() { shareToCalendar(currentEvent); });
	shareButton.addEventListener('click', function(event) {
		showShareControls(); 
	});
	
	return tableFooterView;
}

tableView.addEventListener('click', function() { });


function showShareControls()
{
	shareBlockView.opacity = 0;
	shareBlockView.visible = true;
	shareBlockView.animate(Ti.UI.createAnimation({ opacity: 0.7, duration: 250 }));
	
	shareView.bottom =- shareView.height; 
	shareView.visible = true;
	shareView.animate(Ti.UI.createAnimation({ bottom: 0, duration: 250, }));
}

function hideShareControls() {
	if(shareView.visible)
	{
		var shareAnim = Ti.UI.createAnimation({ bottom: -shareView.height, duration: 250, });
		shareAnim.addEventListener('complete', function(){
			shareView.visible = false;
		});
		shareView.animate(shareAnim);
	}

	if(shareBlockView.visible)
	{
		var blockAnim = Ti.UI.createAnimation({ opacity: 0, duration: 250,  });
		blockAnim.addEventListener('complete', function(){
			shareBlockView.visible = false;
		});
		shareBlockView.animate(blockAnim);
	}
}


win().add(tableView);

function loadEventFromData(data)
{
	// If the event has a latitude supplied, show the map button
	currentEvent = data;	
	var hasMapData = (currentEvent != null && currentEvent.coordinates != null &&
		currentEvent.coordinates.lat != null && currentEvent.coordinates.lat != 0);
	
	if(isIOS()) {
		setNavBarMap(hasMapData, false, clickEventMapButton);
	}
	
	var section = new TableViewSection({ });


	if(isIOS()) {
		section.separatorStyle = Ti.UI.iPhone.TableViewSeparatorStyle.SINGLE_LINE;
	}

	if(data.photos != null)
	{
		var imageRow = Ti.UI.createTableViewRow({
			height:160
		});
		addEventImages(data.photos);
		imageRow.add(scrollView);
		section.add(imageRow);
	}
	
	section.add(makeEventTitleRow(tableView, data.title));
	
	var fullLocation = getEventLocation(data);

	if (!emptyString(fullLocation)) {
		section.add(makeEventDetailRow(tableView, 'Location', fullLocation, null));
	}
	
	//if(hasMapData && isAndroid() && !win().hideMapControl) {
	if(hasMapData && isAndroid()) {
		if (win().hideMapControl) {
			section.add(makeEventMapRow(tableView, launchNativeMap));
		} else {
			section.add(makeEventMapRow(tableView, clickEventMapButton));
		}
	}
	
	var dateString = displayDate(data.date.start, data.date.end);		
	section.add(makeEventDetailRow(tableView, 'Dates', dateString, null));
	
	if (!emptyString(data.web)) {
		section.add(makeEventDetailRow(tableView, 'Website', data.web, clickWebLink));
	}
	
	if (!emptyString(data.phone)) {
		// iPad doesn't support phone links, so make it non-clickable
		section.add(makeEventDetailRow(tableView, 'Telephone', data.phone, (isIPad() ? null : clickPhoneLink)));
	}
	
	if(!emptyString(data.email)) {
		section.add(makeEventDetailRow(tableView, 'Email', data.email, clickEmailLink));
	}

	var categories = '';
	for(var c in data.category)
	{
		categories += data.category[c].name + '\r\n';
	}
	if(!emptyString(categories)){
		section.add(makeEventDetailRow(tableView, 'Categories', categories, null));
	}
	
	var eventBodyLabel = new Label({
		text: data.body,
		height:'auto',
		textAlign: 'left',
		width: getTableContentWidth(win()),
		top:10,
		bottom: 10,
		left:13,
		font: { fontSize: 13 },
		backgroundColor: 'transparent',
		color: app.textColor,
	});

	var eventBodyRow = new TableViewRow({
		width: tableView.width,
		height:'auto',
		backgroundColor:'white',
		className: 'eventBodyRow',
	});
	
	if(isIOS()) {
		eventBodyRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
	}
	
	eventBodyRow.add(eventBodyLabel);
	addFakeSeparator(eventBodyRow);
	section.add(eventBodyRow);

	if(isIOS()) {
	    var shareRow = new TableViewRow({
	        width:'auto',
	        height:'auto',
	        backgroundColor:'white',
	        selectionStyle: Ti.UI.iPhone.TableViewCellSelectionStyle.NONE,
	        className: 'eventBodyRow',
	    });
	    shareRow.add(makeShareFooter());
	    section.add(shareRow);
	}

	tableView.setData([section]);
	
	if(isAndroid()) {
		// Show the footer last so that it does not appear before data.
		tableView.footerView.visible = true;

		// Scroll not required on iOS because the "loading" message is short enough
		// that it causes the table to be scrolled to the top anyway
		tableView.scrollToTop(0);
	}
}

function addEventImages(photos) {
	var paddingBetween = 10;
	var paddingSide = 15;
	var imageWidth = 180;

	scrollView.contentWidth = (imageWidth * photos.length) + (paddingBetween * (photos.length - 1)) + (2* paddingSide);
	
	for(var i in photos)
	{
		var image = Ti.UI.createImageView({
			image:photos[i].url,
			hires:false,
			borderColor: 'white',
			borderWidth: 5,
			backgroundColor: '#e0e0e0',
			left: paddingBetween,
			top: 0,
			width: imageWidth,
			height: 150,
		});
		scrollView.add(image);
	}
}

var shareBlockView = new View({
	backgroundColor: '#333',
	opacity: 0.7,
	width: '100%',
	height: '100%',
	visible: false,
});

shareBlockView.addEventListener('click', function(event){
	hideShareControls();
});

win().add(shareBlockView);

var shareView = new View({
	backgroundColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	bottom: 0,
	width:'100%',
	height: 120,
	opacity: 0.9,
	visible: false,
})

var titleLabel = new Label({
	text: 'Sharing options',
	backgroundColor: 'transparent',
	top: 2,
	left: 10,
	font: { fontSize: 15, fontWeight: 'bold', },
	height: 25,
	color: 'white',
});
shareView.add(titleLabel);

// Add handlers so that if the user is logged in / logged out the displayed 
// controls can be updated
Ti.Facebook.addEventListener('login', handleFacebookLoginChange);
Ti.Facebook.addEventListener('logout', handleFacebookLoginChange);

function handleFacebookLoginChange(e)
{
	if(e != null && e.cancelled === true)
	{
		// If the user has closed the login window, hide the blocker
		hideLoadingWindow();
		setShakeEventsEnabled(true, 'facebook login cancelled');
	}
	else if(awaitingFacebookPost)
	{
		if(Ti.Facebook.loggedIn)		
		{
			showLoadingWindow();
			showFacebookEditingDialog(currentEvent);
		}
	}
	else if(!Ti.Facebook.loggedIn)
	{
		// TODO?
		hideLoadingWindow();
		setShakeEventsEnabled(true, 'facebook no longer logged in');
	}
	awaitingFacebookPost = false;
}


// Create the "share" option views
var shareViews = [];
shareViews.push(createShareView('Twitter', imagePath('share-tweet.png'), function() { shareToTwitter(currentEvent); }));
shareViews.push(createShareView('Email link', imagePath('share-email.png'), function() { shareToEmail(currentEvent); }));
shareViews.push(createShareView('Facebook', imagePath('share-facebook.png'), function() { shareToFacebook(currentEvent); })); 

var closeButton = new View({
	width:25,
	height: 25,
	top: 5,
	right: 5
});

if(isAndroid()) {
	closeButton.width = 40;
	closeButton.height = 30;
}

closeButton.add(new View({
	width:15,
	height:15,
	backgroundImage: imagePath('close-white.png'),
}));

closeButton.addEventListener('click', function(event) {
	hideShareControls();
});

shareView.add(closeButton);

// Calculate the amount of padding to put between each side of the screen, and
// between each item, e.g.:       | pad [item] pad [item] pad [item] pad | 
var totalSharesWidth = 0;
for(var i in shareViews) { totalSharesWidth += shareViews[i].width; }
var padding = ((isIOS() ? win().width : getScreenWidth()) - totalSharesWidth) / (shareViews.length + 1);


// Apply the padding to position the items
for(var left = padding, i = 0; i < shareViews.length; left += shareViews[i].width + padding, i++)
{
	shareViews[i].left = left;
	shareView.add(shareViews[i]);
}

win().add(shareView);

// Perform the startup functions when the window is loaded
(function(){
	
	disableInteractivity();
	
	// Load the specified event from the window eventId parameter, or load a 
	// random event if this is set to the string 'random'
	loadRemoteEvent(win().eventId);
	
})();


function disableInteractivity()
{
	// Remove the map button. Some events do not hav a lat/lng supplied
	if(isIOS()) {
		win().rightNavButton = null;
	}
	hideShareControls();
	showLoading(tableView);
}

var window = win();

var shakeEventsEnabled = false;

function setShakeEventsEnabled(value, description)
{
	Ti.API.debug('shakeEventsEnabled: setting to ' + value + ' due to ' + description);
	shakeEventsEnabled = value;
}

window.addEventListener('focus', function() { setShakeEventsEnabled(true, 'window focus'); });

window.addEventListener('blur', function() { setShakeEventsEnabled(false, 'window blur'); });

window.addEventListener('close', function(e) {
	if(isIPad()) {
		// If this window was opened but will use the back button to close just remove this screen from 
		// the trackable array of screens.
		if(Ti.UI.currentWindow.untrackable) {
			removeLastWindowOnSplitViewStack();
		}
	}
});

/*
// MT: Possible shake fix? 
Ti.Accelerometer.addEventListener('update', accelerometerHandler);

// Have to remember to remove this when the window unloads otherwise
// it will continue to drain the battery
Ti.Accelerometer.removeEventListener('update', accelerometerHandler);


var timestampAccTmp = 0;
var timeResolution = 200;
function accelerometerHandler(e)
{
    timestampAcc = e.timestamp;
    diffTime = timestampAcc - timestampAccTmp;
    
    if(diffTime > timeResolution)
    {
        violence = 1;
        shake = false;
        
        xtop = Math.abs(e.x);
        ytop = Math.abs(e.y);
        ztop = Math.abs(e.z);
        
        //Titanium.API.debug(xtop+" "+ytop+" "+ztop);
        
        if (xtop > violence * 1)
        {
            shake = true;
        }
        if (ytop > violence * 2)
        {
            shake = true;
        }
        if (ztop > violence * 3)
        {
            shake = true;
        }
        if (shake) 
        {
            alert("shake detected "+xtop+" "+ytop+" "+ztop);
        } 
        timestampAccTmp = timestampAcc;
    }
}


*/

// ================================================================================================

function shareToEmail(currentEvent) {

	setShakeEventsEnabled(false, 'email sharing dialog shown');

	var bodyBlurb = null;
	if(currentEvent.body != null)
	{
		bodyBlurb = currentEvent.body;
		if(bodyBlurb.length > 350)
		{
			var pos = 
			bodyBlurb = bodyBlurb.substring(0, bodyBlurb.lastIndexOf(' ', 350)) + '...';
		}
	}
	
	var body = joinStrings('\r\n', [ currentEvent.title, getEventLocation(currentEvent), bodyBlurb ]);
	
	if(currentEvent.url != null) body += '\r\n\r\nFor more information, please visit ' + currentEvent.url + '\r\n';
	
	var dialog = Ti.UI.createEmailDialog({
		subject: currentEvent.title,
		messageBody: '\r\n\r\n' + body,
	});
		
	dialog.addEventListener('complete', function() { setShakeEventsEnabled(true, 'email sharing dialog complete'); });
	
	dialog.open();
}
