Ti.include('../redux.js');

var locations = win().eventLocations;
var pins = [];
var regionLatitude = locations[0].lat;
var regionLongitude = locations[0].lng;
var selectedAnnotation = null;

var linkToNativeMaps = win().linkToNativeMaps;

function addPin(lat, lng, title, subtitle, optionalEventId)
{
	var arrow;
	if(linkToNativeMaps || optionalEventId != null) {
		arrow = rightView;
		arrow.eventId = optionalEventId || 0;
		arrow.latitude = lat;
		arrow.longitude = lng;
		arrow.title = title || 'Unnamed event'
	} else {		
		arrow = null;
	}
	
	var eventPin = Titanium.Map.createAnnotation({
		latitude:lat,
		longitude:lng,
		title:title || 'Unnamed event',
		subtitle: subtitle,
		eventId: optionalEventId || 0,
		pincolor: isAndroid() ? "orange" : Titanium.Map.ANNOTATION_RED,
		animate:true,
		rightView: arrow,
		eventId: optionalEventId || 0,
		latitude:lat,
		longitude:lng,
	});
	
	if(win().linkToEvents && isAndroid()) {
		eventPin.rightButton = coloredImagePath('arwlink.png');
	}
	
	pins.push(eventPin);

}

function clickListButton(event) {
	unloadCurrentWindow();
}

var rightView = new View({
	width: 25,
	height: 25,
	backgroundImage: coloredImagePath('arwlink.png'),
})

if(isIOS()) {
	setNavBarList(clickListButton);
	win().backButtonTitle = "";
}

var includeEventId = (win().linkToEvents === true) || (win().linkToNativeMaps === true);

for(i = 0; i < win().eventLocations.length; i++) {
	var data = win().eventLocations[i];
	addPin(data.lat, data.lng, data.title, data.subtitle, includeEventId ? data.eventId : null);
}

var mapview = Titanium.Map.createView({
	mapType: Titanium.Map.STANDARD_TYPE,
	animate:true,
	regionFit:true,
	userLocation:true,
	region:{latitude:regionLatitude, longitude:regionLongitude, latitudeDelta:0.5, longitudeDelta:0.5},
	annotations:pins
});

win().add(mapview);

rightView.addEventListener('click', function(event) {
	if(selectedAnnotation != null) {
		showEventDetails(selectedAnnotation.eventId, selectedAnnotation.title, selectedAnnotation.latitude, selectedAnnotation.longitude);
	}
});


function showEventDetails(eventId, title, lat, lng)
{
	if(eventId == null)
	{
		Ti.API.warn('showEventDetails called with a null annotation');
	}
	else
	{	
		if(linkToNativeMaps)
		{
			launchBrowser('http://maps.google.com/maps?t=m&q=' + Ti.Network.encodeURIComponent(title) + '@' + lat + ',' + lng);
		}
		else
		{
			viewEventWithId(eventId, true, (isAndroid()? true : false));
		}
	}
}

$(mapview).click(function(event){
	Ti.API.debug('Map clicked. Event: ' + JSON.stringify(event));
	selectedAnnotation = event.annotation;	
	Ti.API.debug('-- Selected event: ' + JSON.stringify(selectedAnnotation));
	Ti.API.debug('   -- The selected event is ' + selectedAnnotation.eventId);
	
	if(selectedAnnotation != null && isAndroid()) {
		Ti.API.info(">> You have selected an event from map list.")
		showEventDetails(selectedAnnotation.eventId, selectedAnnotation.title, selectedAnnotation.latitude, selectedAnnotation.longitude);
	}	
});