Ti.include('../redux.js');

/* Overridable properties (if not set, getSelectedDate/setSelectedDate will be used):
 *   getDate() -> returns the currently selected date
 *   setDate(date) -> sets the currently selected date 
 */
var w = win();

var viewingDate = getSelectedDate();
function getViewingDate() { return viewingDate; }
function setViewingDate(date) { viewingDate = date; }

var tableRows = [];
var calendarView = calendarHeaderView(getPrimaryFilter(), false, true, true, getViewingDate, setViewingDate);
var selectedCategoryLabel = null;

if(isCategorySelected()) {
	var categoryFilterRow = makeCategoryFilterRow(addCategoryFilterAndWriteFilter);
	var selectedCategoryLabel = null;
	tableRows.push(categoryFilterRow);
}

win().add(calendarView);
win().layout = 'vertical';

var tableView = makeFullscreenTable(calendarView.height);
win().add(tableView);

$(win()).focus(function(){ 

	Ti.API.info(">>> Filter by date window has FOCUS!");

	if(isCategorySelected()) {
		var selectedCategory = getSelectedCategory();
		
		Ti.API.info(">>> Set a category to " + selectedCategory);
		
		if (selectedCategory != null) {
			Ti.API.info(">>> Making a category label");
			selectedCategoryLabel = makeRemovableLabel(selectedCategory, removeCategoryFilter, false);
			selectedCategoryLabel.show();
			selectedCategoryLabel.children[0].text = selectedCategory.name;
			categoryFilterRow.add(selectedCategoryLabel);
			categoryFilterRow.children[0].hide();
		}
		else
		{
		}
	}

	refreshTableView();
});

// Add swipe detection to the window. Note that the swipe detection
// is tracked and duplicate events are prevented from firing for a second
// to prevent the event firing twice in a row (TI bug?)
var swipeDirection = 'none';
tableView.addEventListener('swipe', function(event){
	if(event.direction != swipeDirection)
	{
		swipeDirection = event.direction;
		if(event.direction === 'left')
		{
			viewingDate = nextMonth(viewingDate);
			refreshTableView();
		}
		else if(event.direction === 'right')
		{
			viewingDate(previousMonth(viewingDate));
			refreshTableView();
		}
		
		setTimeout(function() { swipeDirection = 'none'; }, 1000);
	}
});

// Setup callbacks
function chooseDate(event) {
	Ti.API.trace('chooseDate(' + JSON.stringify(event) + ')');
	
	newDate = new Date(event.source.id);

	Ti.App.fireEvent('filterCriteriaChanged', { isWholeMonth: false });
	if(win().returnOnSelect == true) {
		setSharedPropertyAsString(win().returnValueKey, newDate);
	} else {
		setSelectedDate(newDate);
	}
	unloadCurrentWindow();
}

function chooseAllMonth(event) {
	Ti.API.trace('chooseAllMonth(' + JSON.stringify(event) + ')');

	newDate = new Date(event.source.id);

	Ti.App.fireEvent('filterCriteriaChanged', { isWholeMonth: true });
	if(win().returnOnSelect == true) {
		setSharedPropertyAsString(win().returnValueKey, newDate);
	} else {
		setSelectedDate(newDate);
	}
	unloadCurrentWindow();
}

function refreshTableView() {
	Ti.App.fireEvent('updateCalendarHeader');	
	showLoading(tableView);
	loadRemoteMonth(tableView, viewingDate.getMonth(), viewingDate.getFullYear(), win().includeAllMonth);
}
