Ti.include('../redux.js');

resetSelectedDateForSection('location_main');

var locationWrapper = {

	/*
	 * Activity indicator - not used on Android, so null on that platform. If not
	 * null, should be added to the "Nearby" row
	 */ 
	activityIndicator : (isAndroid() ? null : Titanium.UI.createActivityIndicator({
		width: 30,
		height: 30,
		right: 10,
		color: '#000',
		style: Ti.UI.iPhone.ActivityIndicatorStyle.DARK,
	})),
	
	/*
	 * Shows the activity indicator, if the platform supports it
	 */
	showActivityIndicator : function()
	{
		if(this.activityIndicator != null)
			this.activityIndicator.show();
	},
	
	/*
	 * Hides the activity indicator, if the platform supports it
	 */
	hideActivityIndicator: function()
	{
		if(this.activityIndicator != null)
			this.activityIndicator.hide();
	},
	
	/*
	 * Whether a lookup is currently in progress
	 */
	isLookingUp: false,
	
	/*
	 * Whether the last lookup failed - to track if two lookups in a row fail so that
	 * the messagebox can be shown to the user, and to prevent it from being shown
	 * on the first request (on page load)
	 */
	lastLookupFailed: false,
	
	/*
	 * The looked-up location name
	 */
	locationName: null,

	callback: function(success, data)
	{
		if(success)
		{
			this.locationName = data.name;
			currentLocationLabel.text = data.name;
			// also has data.lat and data.lng
			this.hideActivityIndicator();
			cancelLoadingIndicator();
			nearbyArrow.show();

			
			if(isIOS()) {
				nearbyRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.GRAY;
			} else {
				nearbyRow.backgroundSelectedColor = '#999';
			}
		}
		else
		{
			
			Ti.API.info(">>> Location lookup failed");
			nearbyArrow.hide();
			this.locationName = null;
			currentLocationLabel.text = '';
			this.hideActivityIndicator();
			cancelLoadingIndicator();
			if(isIOS()) {
				nearbyRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
			}
		
			if(this.lastLookupFailed === true)
			{
				messageBox('Please ensure Location Services is enabled for Vic Events in your phone Settings.');
			}
	
		}
		
		// Set this last so that the first request can fail silently
		this.isLookingUp = false;
		this.lastLookupFailed = !success;
	},
	
	/**
	 * Returns true if the request is pending, false if location services is turned off
	 */
	startLookup : function() 
	{
		this.showActivityIndicator();
		this.isLookingUp = true;
	
		if(isIOS()) {
			nearbyRow.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
		}
	
		clearLastLatLng();
		
		Ti.API.info(">>> Testing location services: " + Titanium.Geolocation.locationServicesEnabled);
		
		if (Titanium.Geolocation.locationServicesEnabled === false) {
			Ti.API.info(">>> Location services is disabled...");
			this.callback(null, null);
			return false;
		}
		
		if (isIOS()) {
			Titanium.Geolocation.purpose = "Retrieve your local area.";
		}
		
		var thisWrapper = this;
		
		Titanium.Geolocation.getCurrentPosition(function(e){
			
			if (e.error) {
				thisWrapper.callback(false, e.error);
				return;
			} else {
				longitude = e.coords.longitude;
				latitude = e.coords.latitude;
				
				setLastLatLng(latitude, longitude);
	
				var url="http://maps.google.com/maps/api/geocode/json?latlng="+latitude+","+longitude+"&sensor=true";
				var xhr = Titanium.Network.createHTTPClient();
				xhr.open('GET',url);
				
				xhr.onload = function(label) {
					var json = this.responseText;
				    var gotitems = eval('(' + json + ')');
				    
				    var name = gotitems.results[1].formatted_address.split(",")[0];
				    
				    thisWrapper.callback(true, {
				    	success: true,
				    	lat:  e.coords.latitude,
				    	lng: e.coords.longitude,
				    	name: name,
				    })
			    };
			
				xhr.send();			
			}	
		});
		
		return true;
	}

};

function filterByCouncil(event) {
	loadWindow(new Window(mixinWindowOptions({
	    id: 'win_view_by_council',
	    url: 'filter_by_council.js',
		title:'View by local council',
		barColor: win().barColor,
	})));
}

function filterByRegion(event) {
	loadWindow(new Window(mixinWindowOptions({
	    id: 'win_view_by_region',
	    url: 'filter_by_region.js',
		title:'View by region',
		barColor: win().barColor,
	})));
}




var tableView = makeFullscreenTable();
var tableRows = [];
var nearbyArrow = makeHasChildIndicator();

// Setup reverse geocode
var nearbyRow = new TableViewRow({hasChild:false,height:39,backgroundColor:'#fff', backgroundSelectedColor:'#999', width: tableView.width});
var titleLabel = new Label({text:'Nearby', left:10, font: {fontSize:13, fontWeight:'bold',}, width:200, height:30, backgroundColor:'transparent', color: app.headerColor, });
var currentLocationLabel = new Label({text:'', color: app.tipTextColor, backgroundColor:'transparent', textAlign:'right',right:30, width:200, visible: true, ellipsize:true, font: { fontSize: 13, } } );
nearbyRow.add(titleLabel);
nearbyRow.add(currentLocationLabel);
nearbyRow.add(nearbyArrow);
nearbyRow.id = 0;
tableRows.push(nearbyRow);

if(isIOS())
{
	nearbyRow.add(locationWrapper.activityIndicator);
}

startLookingUpLocation();

var r1 = makeLinkedStyledRow(1, "Browse by local council", null, null, filterByCouncil);
addFakeSeparator(r1);
tableRows.push(r1);

var r2 = makeLinkedStyledRow(2, "Browse by region", null, null, filterByRegion);
addFakeSeparator(r2);
tableRows.push(r2);

tableView.data = tableRows;
if(isAndroid()) {
	tableView.top = win().top;
}

win().add(tableView);

// Callback click handlers
$(nearbyRow).click(function(event){
	if(currentLocationLabel.text != '')
	{
		// TODO: If we don't know the location, ask permission to get it
		
		clearFilters();
		resetSelectedDateForSection('nearby');

		setPrimaryFilter(0, currentLocationLabel.text);
		
		loadEventsWindow('Nearby', false, true);
	}
	else if(locationWrapper.isLookingUp === false)
	{
		startLookingUpLocation();
	}
});


function startLookingUpLocation()
{
	currentLocationLabel.text = '';
	nearbyArrow.hide();
	locationWrapper.startLookup();
}
