Ti.include('../redux.js');

Ti.API.trace('Starting home.js');

var clearFiltersFlag = true;

var searchBar = new SearchBar({
	id: 'search_bar',
	hinttextid:'search_bar_hint',
	barColor:'#002E42',
	height: 60,
	top: 0,
	zindex:999
});

if(isAndroid()) {
	searchBar.showCancel = false;
	searchBar.keyboardType = Titanium.UI.KEYBOARD_DEFAULT;
	searchBar.returnKeyType = Titanium.UI.RETURNKEY_DONE; // This does not seem to take effect on Android.
}

win().add(searchBar);

function clearSearchBox()
{
	searchBar.value = '';
}

var background = new View({
	height: 356,
	width: 288,
	top: searchBar.height + 10,
	backgroundImage: imagePath('menu.png'),
	left: 14,
});

win().add(background);

searchBar.addEventListener('return', function() {
	if(emptyString(searchBar.value))
	{
		messageBox(L('search_query_empty'));
		return;
	}

	hideSearchKeyboard();
	setColors('#009dc9', '#044456','blue');
	
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_view_search_results',
	    url: 'view_search_results.js',
		title:'Search results',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
		searchQuery: searchBar.value,
	}));
	
	loadWindow(newWin);
});


// View by Date
var viewByDateView = new View({top:10,left:17,height:143,width:117, backgroundColor:'transparent'});
background.add(viewByDateView);

$(viewByDateView).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();
	setColors('#bf5f16', '#7c3e0e','red');

	removeSelectedCategory();
	resetSelectedDateForSection('date');
	
	loadEventsWindow('View by date', true, false);
});

// View by Location
var viewByLocationView = new View({top:5,left:145,backgroundColor:'transparent',height:165,width:130});
background.add(viewByLocationView);

$(viewByLocationView).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();
	setColors('#f0af13', '#ca8d00','orange');

	var newWin = new Window(mixinWindowOptions({
	    id: 'win_filter_by_location',
	    url: 'filter_by_location.js',
		title:'View by location',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	}));
	
	loadWindow(newWin);	
});

// View by Category
var viewByCategoryView = new View({top:165,left:35,backgroundColor:'transparent',height:160,width:130});
background.add(viewByCategoryView);

$(viewByCategoryView).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();
	setColors('#6cb640', '#4c8608','green');

	resetSelectedDateForSection('categories');
	
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_view_by_category',
	    url: 'filter_by_category.js',
		title:'View by category',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR)
	}));

	loadWindow(newWin);
});

// View Random
var viewRandomView = new View({left:170,top:175,backgroundColor:'transparent',height:125,width:110});
background.add(viewRandomView);

$(viewRandomView).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();
	setColors('#009dc9', '#044456','blue');

	resetSelectedDateForSection('random');

	var newWin = new Window(mixinWindowOptions({
	    id: 'win_event_details',
	    url: 'event.js',
		title:'Event',
		eventId:'random',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
		showShakerBar: true,
	}));
	
	loadWindow(newWin);
});

// Suggest an Event
var suggestButton = new Button({
	backgroundImage: imagePath('turquoise/btn.png'),
	font: { fontSize: 15, fontWeight: 'bold', },
	width: 170,
	height: 44,
	right:20,
	bottom: 20,
})
if(isIOS()) {
	suggestButton.title = 'suggest an event';
	suggestButton.backgroundImage = imagePath('turquoise/btn.png');
} else {
	suggestButton.width = 169;
	suggestButton.height = 40;
	suggestButton.backgroundImage = imagePath('turquoise/btn_suggest.png');
}
win().add(suggestButton);

$(suggestButton).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();
	setColors('#00AF9D', '#014A42', 'turquoise'); //'#014A42'
	//setColors('#f0af13', '#946417','orange');

	var options = mixinWindowOptions({
	    id: 'win_suggest_an_event',
	    url: 'suggest_event.js',
		title:'Suggest an event',
		layout:'vertical',
		barColor: getSharedPropertyAsString(appProps.KEY_FILTER_BAR_COLOR),
	});

	var newWin = new Window(options);
	loadWindow(newWin);
});

function hideSearchKeyboard()
{
	if(isIOS()) {
		searchBar.blur();
	}
}

// Whenever the window gains focus (e.g. coming back from another window)
// hide the search bar keyboard if it is shown
if(isIOS()) {
	Ti.UI.currentWindow.addEventListener('focus', function() {
		hideSearchKeyboard();
		clearFilters();
		clearPrimaryFilter();
	});
}

if(isAndroid()) {
	Titanium.Android.currentActivity.addEventListener("start", function(){
		hideSearchKeyboard();
		clearFilters();
		clearPrimaryFilter();
	});
}

win().addEventListener('click', hideSearchKeyboard);

// View by Category
var aboutView = new View({
	bottom: 18,
	left:15,
	backgroundColor:'transparent',
	height:53,
	width:45,
	backgroundImage: imagePath('menu-info.png'),
});
win().add(aboutView);

$(aboutView).click(function() { 
	hideSearchKeyboard();
	clearSearchBox();

	var newWin = new Window({
	    id: 'win_about',
	    url: 'about.js',
		title:'About',
		layout:'vertical',
		backgroundColor: 'white',
		backgroundImage: null,
		navBarHidden: false,
	});
	
	if(isAndroid()) {
		newWin.orientationModes = [Ti.UI.PORTRAIT, Ti.UI.UPSIDE_PORTRAIT];
	}

	newWin.open({modal:true});
});


function doOnlineCheck(forceFail)
{
	// Perform the online check
	if(!Ti.Network.online || forceFail === true)
	{
		if(isAndroid()) {
			//loading.show();
		} else {
		
			showLoadingWindow();
		
			var w = Titanium.UI.createWindow({
				backgroundColor:'#eee',
				borderWidth:1,
				borderColor:'#eee',
				height:200,
				width:250,
				borderRadius:10,
			});	

			var lbl = new Label({
				width: 200,
				top: 35,
				height: 'auto',
				backgroundColor: 'transparent',
				opacity: 1,
				textAlign:'center',
				text: 'You must have an\ninternet connection to\nuse Vic Events.',
			});
			
			w.add(lbl);
			
			var btn = new Button({
				width: 200,
				height: 40,
				bottom: 35,
				title: 'Retry',
				backgroundImage: imagePath('turquoise/btn-dark.png'),
			})
			
			w.add(btn);
		
			bounceOpen(w);
		
			btn.addEventListener('click', function() {
				if(!Ti.Network.online)
				{
					messageBox('Sorry, no internet connection was detected.');
				}
				else
				{
					hideLoadingWindow();
					var t3 = Titanium.UI.create2DMatrix();
					t3 = t3.scale(0);
					w.close({transform:t3,duration:300});
				}
			})
		}
	
	}
}


Ti.UI.currentWindow.addEventListener('focus', function(){ doOnlineCheck(); });
