Ti.include('../redux.js');

function clickCancelButton()
{
	Ti.API.trace('cancel button clicked');
	if(currentStep > 0)
	{
		var dialog = Ti.UI.createAlertDialog({
			title: null,
			message: 'Any information you have entered will be discarded. Are you sure you want to cancel?',
			buttonNames: ['Yes','No']
		});
		
		dialog.addEventListener('click', function(e){
			if(e.index == 0) {
				if(isIPad())
				{
					currentStep = 0;
					showPage();
				}
				else
				{
					win().close();
				}
			}
		});
		
		dialog.show();
	}
	else
	{
		if(!isIPad())
		{
			win().close();
		}
	}
}

/*
 * Returns a primary (turquoise) button.
 * @param title:string Optional. The button title. Defaults to "next"
 * @param callback:function Optional. The function to run when the button is clicked. Default behaviour checks mandatory fields are entered then increments currentPage and displays the new page.
 */
function makeNextButton(title, callback)
{
	if(title == null) { title = 'next'; }
	
	if(callback == null)
	{
		callback = function(){
			page.hideKeyboard();
			
			if(page.validateFunction != null)
			{
				if(!page.validateFunction())
					return;
			}
			
			page.saveTextFieldValues();
			currentStep++;
			showPage();
		};
	}
	
	var button = new Button({ height:40, top: 0, right: 10, width:120, color:'#FFF', font: { fontSize: 15, fontWeight: 'bold', }, backgroundLeftCap:10, backgroundTopCap:5 });
	
	if(isIOS()) {
		button.title = title;
		button.backgroundImage = imagePath('turquoise/btn.png');
	} else {
		button.image = imagePath('turquoise/btn_' + title + '.png');
		button.width = 140;
		button.height = 40;
	}
	
	button.addEventListener('click', callback);
	return button;
}

function makePrevButton()
{
	return makeSecondaryButton('previous', function() {
		page.hideKeyboard();
		page.saveTextFieldValues();
		currentStep--;
		showPage();
	});
}

function makeSecondaryButton(text, callback)
{
	var button = new Button({ height:40, top: 0, left: 10, width:120, font: { fontSize: 15, fontWeight: 'bold', }, });
	
	if(isIOS()) {
		button.title = text;
		button.backgroundImage = imagePath('turquoise/btn-dark.png');
	} else {
		button.image = imagePath('turquoise/btn_' + text + '.png');
		button.width = 140;
		button.height = 40;
	}	
	
	button.addEventListener('click', callback);
	return button;
}

//========================================================================================================

var page = {

	// Required to hold the label and table due to Android not properly supporting
	// remove on Window objects in 1.7.x
	topContainer: null,

	topLabel: null,
	topTable: null,

	currentPage: null,
	previousPage: null,
	
	title: null,
	leftButton: null,
	rightButton: null,
	fullButton: null,
	
	validateFunction: function() { return false; },
	
	fields: [],
	focusableFields: [],
	selectedField: null,

	onFieldFocus: function(e)
	{
		page.selectedField = e.source;
		Ti.API.trace('onFieldFocus: selectedField is now ' + page.selectedField);
	},
	
	hideKeyboard: function()
	{
		Ti.API.trace('hideKeyboard: selectedField was ' + page.selectedField);
		if(page.selectedField != null)
		{
			page.selectedField.blur();
		}
	},

	addBulletPoint: function(text)
	{
		// Event name
		var lineView = new View({
			height: 'auto',
			top:0,
			width:'100%',
		});
		
		lineView.add(new Label({ top: 0, left:10,  text: '•', width: '10', height: 'auto', color: app.textColor, font: { fontSize: 11 }, }));
		lineView.add(new Label({ top: 0, left:20, text:text, height:'auto', width: 'auto', font: { fontSize: 11 }, color: app.textColor, }));		
		
		this.currentPage.add(lineView);
	},

	makeSuggestDateField: function(id){
		
		var field = new Label({ text:'  Tap to select', color:'#AAAAAA', height:35, top:5, left:0, right:0, borderColor:'#999', paddingLeft: 10, id: id, });
		
		if(isIOS()) {
			field.borderColor = '#999';		
			field.borderStyle = Titanium.UI.INPUT_BORDERSTYLE_LINE;
			field.borderRadius = 10;
		} else {
			field.borderWidth = 1;
			field.backgroundColor = '#FFF';
		}
		
		return field;
	},

	makeSuggestTextArea: function() {
		var field = new TextArea({ height:70, top:5, left:0, right:0, paddingLeft:10, font: { fontSize: 14 }, });
	
		if(isIOS()) {
			field.borderColor = '#999';	
			field.borderRadius = 10;
		} else {
	
		}
		
		return field;
	},

	makeSuggestDropdown : function(isRemovable)
	{
		var field = new Label({ height:35, top:5, left:0, right:0, borderColor:'#999', paddingLeft: 10, text:'' });
		
		if(isAndroid()) {
			field.borderWidth = 1;		
			field.backgroundColor = '#FFF';
		} else {
			field.borderRadius = 10;	
		}
		
		if(isRemovable)
		{
			//var remover = new View({ height: 15, width: 15, right: 20, backgroundColor: 'red', });
		}
		
		return field;
	},

	makeSuggestTextField: function(optionalKeyboardType)
	{
		var field = new TextField({ height:35, top:5, left:0, right:0, paddingLeft:10, });
		
		if(isIOS()) {
			field.borderColor = '#999';		
			field.borderStyle = Titanium.UI.INPUT_BORDERSTYLE_LINE;
			field.borderRadius = 10;
		}
		
		if(optionalKeyboardType != null)
			field.keyboardType = optionalKeyboardType;
	
		return field;
	},

	/*
	 * Shows the date selection window with the specified title, and stores the
	 * date selected (if any) in to the properties with a key specified by returnValueKey.
	 */
	selectDate: function(title, returnValueKey)
	{
		this.hideKeyboard();
		this.saveTextFieldValues();

		var newWin = new Window(mixinWindowOptions({
		    url: 'list_all_dates.js',
			title: title,
			barColor: win().barColor,
			returnValueKey:returnValueKey,
		}));
		loadWindow(newWin);	
	},

	saveTextFieldValues: function() {
		for(var i in this.fields)
		{
			var field = this.fields[i];
			if(field.dataKey != null)
			{
				if(field.type === 'date' ||
				   field.type === 'dropdown')
				{
					// This is actually set when returning from the selection screen
					// so don't need to store it
				}
				else
				{
					setSharedPropertyAsString(field.dataKey, field.value);
				}
			}
		}
	},

	/*
	 * Tries to load a date object from the property store using the specified keyName, and
	 * writes the friendly date name to the specified field, or a generic "Tap to select"
	 * message if that value has not yet been set.
	 */
	tryLoadDate: function(field, keyName)
	{
		var value = stringOrNull(getSharedPropertyAsString(keyName));
		if(value == null)
		{
			field.color = '#aaa';	
			field.text = '  Tap to select';
		}
		else
		{
			var d = new Date(value);
			field.color = '#000';	
			field.text = "  " + d.getDate() + " " + getMonthName(d.getMonth()) + ", " + getDayName(d.getDay());
		}
	},

	loadTextFieldValues: function(){
		for(var i in this.fields)
		{
			var field = this.fields[i];
			if(field.dataKey != null)
			{
				if(field.type == 'date')
					this.tryLoadDate(field, field.dataKey);
				else if(field.type == 'dropdown')
					this.tryLoadDropdown(field, field.dataKey);
				else
					field.value = getSharedPropertyAsString(field.dataKey);
			}
		}
	},

	tryLoadDropdown: function(field, keyName)
	{
		var value = stringOrNull(getSharedPropertyAsString(keyName));
		
		if(value == null)
		{
			field.color = '#aaa';	
			field.text = '  Tap to select';
		}
		else
		{
			field.color = '#000';	
			field.text = "  " + value;
		}
	},
	
	cancelButton: null,
	
	setCancelButtonVisible: function(visible)
	{
		// No cancel button on iPad!
		if(isIPad())
			return;
			
		if(visible)
		{
			if(this.cancelButton == null)
			{
				Ti.API.trace('setCancelButtonVisible: creating button');
				
				this.cancelButton = new Button({title: 'Cancel'});
				this.cancelButton.addEventListener('click', clickCancelButton);
				win().leftNavButton = this.cancelButton;
			}
		}
		else
		{
			if(this.cancelButton != null)
			{
				Ti.API.trace('setCancelButtonVisible: removing button');

				this.cancelButton = null;
				win().rightNavButton = null;
			}
		}
	},

	/*
	 * @param options An object definining the 'name' for the field label, 'mandatory' (true or false), and type 'type' (text or textarea)
	 */
	addField: function(options)
	{
		var field = null;
		
		this.addFieldName(options.name, options.mandatory)

		switch(options.type)
		{
			case 'date':
				if(options.popupTitle == null)
					Ti.API.warn('addField added a date field but no popupTitle was specified; using name');
					
				field = this.makeSuggestDateField('a_date'); // TODO: remove param?
				field.addEventListener('click', function() {
					page.selectDate(options.popupTitle || options.name, options.dataKey);
				});
				break;

			case 'email':
				field = this.makeSuggestTextField(Ti.UI.KEYBOARD_EMAIL);
				this.focusableFields.push(field);
				break;

			case 'phone':
				field = this.makeSuggestTextField(Ti.UI.KEYBOARD_NUMBERS_PUNCTUATION);
				this.focusableFields.push(field);
				break;

			case 'text':
				field = this.makeSuggestTextField(options.keyboard);
				this.focusableFields.push(field);
				break;
				
			case 'textarea':
				field = this.makeSuggestTextArea(options.keyboard);
				this.focusableFields.push(field);
				break;

			case 'dropdown':
				field = this.makeSuggestDropdown();
				
				field.addEventListener('click', function() {
					page.hideKeyboard();
					
					var newWin = new Window(mixinWindowOptions(options.windowOptions));
					page.saveTextFieldValues();
					loadWindow(newWin);	
				});
				break;
			
			default: alert('Unhandled addField type: ' + options.type); break;

		}
		
		if(field != null)
		{
			field.type = options.type;
			field.displayName = options.name;
	
			if(options.dataKey != null)
				field.dataKey = options.dataKey;
		
			field.mandatory = options.mandatory;
			
			this.currentPage.add(field);
			this.fields.push(field);
			field.addEventListener('focus', this.onFieldFocus);
		}

		return field;
	},

	addFieldName : function(text, isMandatory)
	{
		var width = 'auto';
		
		var eventName = new Label({ text:text, top:10, left:0, height:20, width: width, font: { fontSize: 13 }, color: '#333', });
		
		if(isMandatory)
		{
			var subBox = new View({ width: width, height: eventName.height + eventName.top, backgroundColor: 'transparent', left: 0, })
			
			subBox.add(eventName);
			
			if(isAndroid()) {
				eventName.left = 10;
			}
			
			subBox.add(new Label({ text:'*', top:eventName.top, left:eventName.left + eventName.width + 5, height:'auto', width: width, color: app.headerColor, font: { fontSize: 13 } }));
			this.currentPage.add(subBox);
		}
		else
		{
			this.currentPage.add(eventName);
		}
	},

	addHeading: function(text) {
		this.currentPage.add(new Label({ text:text, color:app.headerColor, backgroundColor:'transparent', font: { fontSize: 16, fontWeight: 'bold', }, top:10, left:0, height:'auto', textAlign:'left' }));
	},
	
	addView : function(view)
	{
		this.currentPage.add(view);
	},
	
	addLabel : function(text, options)
	{
		var label = this.makeLabel(text, options);
		this.currentPage.add(label);
		return label;
	},
	
	makeLabel: function(text, options)
	{
		var masterOptions = { text:text, height:'auto', top:10, left:0, color: app.textColor, font: { fontSize: 11 }, };
		
		if(options != null)
		{
			_.extend(masterOptions, options);
		}

		var label = new Label(masterOptions);
		return label;
	},
	
	addMandatoryReminder: function()
	{
		var width = (isAndroid() ? 'auto' : '100%');

		var subBox = new View({ height: 'auto', top: 5, backgroundColor: 'transparent', width:width });
		subBox.add(new Label({ text:'mandatory field', top:0, left:10, height:'auto', width: width, font: { fontSize: 13 }, color: '#333', }));
		subBox.add(new Label({ text:'*', top:0, left:0, height:'auto', width: width, color: app.headerColor, font: { fontSize: 13 } }));
		this.currentPage.add(subBox);
	},
	
	addSpacer : function(height)
	{
		// TODO: Does android need an equivalent?
		if(isIOS())
		{
			this.currentPage.add(new View({width:5, height:height, backgroundColor: 'transparent',}));
		}
	},

	/*
	 * Resets the fields, focusableFields, selectedField, validateFunction, title, leftButton and rightButton objects.
	 * Moves the currentPage to previousPage, and creates a new empty currentPage (the actual swap happens when you call render()).
	 */
	start: function() {

		Ti.API.trace('start: clearing variables');
		var window = win();
		
		this.previousPage = this.currentPage;

		this.currentPage = new View({
			top:0, width: getTableContentWidth(window), layout:'vertical',
			height: 'auto', // MT: was a statement after the constructor
		});

		// Not sure why, but unless you put a listener on the container,
		// events don't bubble up to the field handlers...
		this.currentPage.addEventListener('click', function() {});
		
		this.fields = [];
		this.focusableFields = [];
		this.selectedField = null;
		
		this.validateFunction = function(){ return page.validate(); }; // No special validation required except checking mandatory fields are present.
		this.title = null;
		this.leftButton = null;
		this.rightButton = null;

	},
	
	render: function() {
		var window = win();
		window.layout = 'vertical';
		
		if(isAndroid())
		{
			Ti.UI.Android.hideSoftKeyboard();
		}
		
		if(this.topContainer == null)
		{
			this.topContainer = new View({
				width: getScreenWidth(), 
				layout: 'vertical',
				height: isAndroid() ? (getScreenHeight() - 50) : getWindowHeight(window),
			});
			
			window.add(this.topContainer);
		}
		else
		{
			if(this.topLabel != null) { this.topContainer.remove(this.topLabel); this.topLabel = null; }
			if(this.topTable != null) { this.topContainer.remove(this.topTable); this.topTable = null; }
		}

		if(this.title != null)
		{
			var stepView = new View({ top:0, height:40, backgroundColor: getSharedPropertyAsString(appProps.KEY_FILTER_DARK_BAR_COLOR) || '#AAA' });
			var stepLabel = new Label({ text:this.title, height:'auto', font: { fontSize: 14, fontWeight:'bold' }, textAlign:'center', color:'#FFFFFF', backgroundColor:'transparent' });
			stepView.add(stepLabel);
			this.topLabel = stepView;
			this.topContainer.add(this.topLabel);
		}
		
		this.topTable = makeFullscreenTable(this.topLabel != null ? this.topLabel.height : 0);

		var row = new TableViewRow({ backgroundColor: '#fff', height:'auto' });
		
		if(!isAndroid()) {
			row.selectionStyle = Ti.UI.iPhone.TableViewCellSelectionStyle.NONE;
			row.height = 'auto';
		}
		
		row.add(this.currentPage);

		var buttonView = new View(isAndroid() ? { backgroundColor: 'white', height: 65 + (Ti.Platform.displayCaps.platformHeight / 2), top: 40, } : { height:50, top:20, });

		if(this.fullButton != null)
		{
			this.fullButton.left = 0;
			this.fullButton.right = 0;
			this.fullButton.width *= 2;
			buttonView.add(this.fullButton);
		}
		else
		{
			if(isIPad()) {
				if(this.leftButton != null)
					this.leftButton.left = 45;
				
				if(this.rightButton != null)
					this.rightButton.right = 45;
			}

			if(this.leftButton != null)
				buttonView.add(this.leftButton);
			
			if(this.rightButton != null)
				buttonView.add(this.rightButton);
				
		}
		this.loadTextFieldValues();
		
		row.addEventListener('click', function(evt) { Ti.API.debug('row click handler fired - bubble events?') });
		
		this.topTable.footerView = buttonView;
		this.topTable.setData([row]);
		this.topContainer.add(this.topTable);

		Ti.API.trace('render complete');
	},
	
	/*
	 * Returns either a Date object from the specified property key, or null.
	 */
	getValidSuggestDate: function(appPropsKey)
	{
		var str = getSharedPropertyAsString(appPropsKey);
		if(emptyString(str))
			return null;
			
		var d = new Date(str);
		if(d != null)
			d.setHours(0,0,0,0);
		return d;
	},

	/*
	 * Validates that the mandatory fields are present. Displays an error message
	 * and returns false if any are missing values, otherwise silently returns true.
	 */
	validate : function ()
	{
		var isValid = true;
		
		var missingFieldNames = [];
		
		for(var i in this.fields)
		{
			var field = this.fields[i];
			if(field.mandatory === true)
			{
				if(field.type == 'date')
				{
					var date = this.getValidSuggestDate(field.dataKey);
					if(date == null)
					{
						missingFieldNames.push(field.displayName);
						isValid = false;
					}
				}
				else if(field.type == 'dropdown')
				{
					if(emptyString(getSharedPropertyAsString(field.dataKey)))
					{
						missingFieldNames.push(field.displayName);
						isValid = false;
					}
				}
				else if(emptyString(field.value))
				{
					{
						missingFieldNames.push(field.displayName);
						isValid = false;
					}
				}
			}	
		}
		
		if(!isValid)
		{
			// A field is missing
			if(missingFieldNames.length > 0)
			{
				var fieldNamesString;
				if(missingFieldNames.length == 1)
					fieldNamesString = missingFieldNames[0];
				else if(missingFieldNames.length == 2)
					fieldNamesString = missingFieldNames[0] + ' and ' + missingFieldNames[1].toLowerCase();
				else
				{
					// Lowercase all except the first value
					for(var i = 1; i < missingFieldNames.length; i++)
					{
						missingFieldNames[i] = missingFieldNames[i].toLowerCase();
					}
					
					fieldNamesString = ' and ' + missingFieldNames.pop();
					fieldNamesString = missingFieldNames.join(', ') + fieldNamesString;
				}
				
				messageBox(String.format(L('error_mandatory_field_named'), fieldNamesString));
			}
			else
			{
				messageBox(L('error_mandatory_fields'));
			}
		}
		else
		{
			for(var i in this.fields)
			{
				var field = this.fields[i];
				if(!emptyString(field.value))
				{
					if(field.type === 'email')
					{
						if(field.value.indexOf('@') == -1)
						{
							messageBox(L('error_invalid_email'));
							return false;
						}
					}
					else if(field.type === 'phone')
					{
						for(var i = 0; i < field.value.length; i++)
						{
							if('1234567890 ()-'.indexOf(field.value[i]) == -1)
							{
								messageBox(L('error_invalid_phone'));
								return false;
							}
						}
						
					}
				}
			}
			
		}
		
		return isValid;
	},
};

// Clear the form data when the screen is loaded (i.e. coming in from the home screen)
for(var i in appProps.ALL_SUGGEST_EVENT_KEYS) {
	Titanium.App.Properties.removeProperty(appProps.ALL_SUGGEST_EVENT_KEYS[i]);
}

// Hide the back button
//if(isIOS()) { win().leftNavButton = new View({width:1, height:1, backgroundColor:'transparent', }); }

var currentStep = 0;
win().layout = 'vertical';
showPage();

function showPage()
{
	page.start();
	
	switch(currentStep)
	{
		case 0:
			page.setCancelButtonVisible(false);
			page.addHeading('Guidelines');
			page.addLabel('We welcome you to submit an event for listing in the online calendar. All events must adhere to the following guidelines:');
			page.addSpacer(5);
			page.addBulletPoint('Publicly accessible');
			page.addBulletPoint('Free or modestly priced');
			page.addBulletPoint('Have a degree of Government support (at local,state or federal levels)');
			page.addBulletPoint('Encourage active community participation');
			page.addBulletPoint('Inclusive and embraces diversity.');
			page.addHeading('Disclaimer');
			page.addLabel(L('suggest_terms_disclaimer'));

			var emailLinkView = Titanium.UI.createView({
				width:200,
				backgroundColor: 'transparent',
				height: 40,
				left:0,
			});
			emailLinkView.add(page.makeLabel(L('suggest_email_address'), { color: app.headerColor, id: 'email_link', top: 5, left: 0, }));
			
			page.addView(emailLinkView);
			emailLinkView.addEventListener('click', function() {
				var emailDialog = Titanium.UI.createEmailDialog();
				emailDialog.toRecipients = [L('suggest_email_address')];
				emailDialog.open();
			});
			
				page.rightButton = makeNextButton('proceed');
			
			if(!isIPad())
			{
				page.leftButton = makeSecondaryButton('cancel', function() { win().close(); });
			}
			break;
			
		case 1:
			page.setCancelButtonVisible(true);
			page.title = 'Step 1 of 4';
			page.addHeading('Event details');
			page.addMandatoryReminder();
			page.addField({ type: 'text', name: 'Event name', mandatory: true, dataKey: appProps.KEY_EVENT_NAME, });
			page.addField({ type: 'date', name: 'Start date', mandatory: true, dataKey: appProps.KEY_EVENT_START_DATE, popupTitle: 'Select start date', });
			page.addField({ type: 'date', name: 'End date', mandatory: true, dataKey: appProps.KEY_EVENT_END_DATE, popupTitle: 'Select end date', });
			page.addField({ type: 'textarea', name: 'Description', mandatory: true, dataKey: appProps.KEY_EVENT_DESCRIPTION, });
			page.addSpacer(15);
			
			page.leftButton = makePrevButton();
			page.rightButton = makeNextButton();

			page.validateFunction = function(){
				Ti.API.trace('Default validateFunction called.');
				page.hideKeyboard();
				if(page.validate())
				{
					// Fields are present, do any extra validation
					var startDate = page.getValidSuggestDate(appProps.KEY_EVENT_START_DATE);
					var endDate = page.getValidSuggestDate(appProps.KEY_EVENT_END_DATE);
					if (endDate < startDate)
					{
						messageBox(L('error_start_date_end_date_order'));
					}
					else
					{
						return true;
					}
				}
				return false;
			};
			break;

		case 2:
			page.setCancelButtonVisible(true);
			page.addHeading('Event location and address');
			page.addField({ type: 'text', name: 'Location', mandatory: false, dataKey: appProps.KEY_EVENT_LOCATION, });
			page.addField({ type: 'text', name: 'Street', mandatory: false, dataKey: appProps.KEY_EVENT_STREET, });
			page.addField({ type: 'text', name: 'Town/suburb', mandatory: false, dataKey: appProps.KEY_EVENT_TOWN, });

			page.addField({ type: 'dropdown', name: 'State', mandatory: false, dataKey:appProps.KEY_EVENT_STATE,
				windowOptions: {
				    url: 'filter_by_state.js',
					title:'Select state',
					returnOnSelect:true,
					returnValueKey:appProps.KEY_EVENT_STATE,
					barColor: win().barColor,
				}});

			page.addField({ type: 'text', name: 'Additional message', mandatory: false, dataKey: appProps.KEY_EVENT_MESSAGE, });
			
			page.addField({ type: 'dropdown', name: 'Council', mandatory: false, dataKey:appProps.KEY_EVENT_COUNCIL,
				windowOptions: {
				    url: 'filter_by_council.js',
					title:'Select council',
					returnOnSelect:true,
					returnValueKey:appProps.KEY_EVENT_COUNCIL,
					barColor: win().barColor,
					isFullList: true,
				}});

			page.addField({ type: 'dropdown', name: 'Region', mandatory: false, dataKey:appProps.KEY_EVENT_REGION,
				windowOptions: {
				    id: 'win_event_region',
				    url: 'filter_by_region.js',
					title:'Select region',
					returnOnSelect:true,
					returnValueKey:appProps.KEY_EVENT_REGION,
					barColor: win().barColor,
					isFullList: true,
				}});
				
			page.addSpacer(15);

			page.title = 'Step 2 of 4';
			page.leftButton = makePrevButton();
			page.rightButton = makeNextButton();
			break;
			
		case 3:
			page.setCancelButtonVisible(true);
			page.addHeading('Event contact details and category(s)');
			page.addMandatoryReminder();
			page.addField({ type: 'phone', name: 'Telephone', mandatory: false, dataKey: appProps.KEY_EVENT_PHONE, });
			page.addField({ type: 'email', name: 'Email', mandatory: false, dataKey: appProps.KEY_EVENT_EMAIL, });
			page.addField({ type: 'text', name: 'Web', mandatory: false, dataKey: appProps.KEY_EVENT_WEBSITE, keyboard: Ti.UI.KEYBOARD_URL });
			
			page.addField({ type: 'dropdown', name: 'Category 1', mandatory: true, dataKey:appProps.KEY_EVENT_CAT1,
				windowOptions: {
				    id: 'win_event_category',
				    url: 'filter_by_category.js',
					title:'Select a category',
					returnOnSelect:true,
					returnValueKey:appProps.KEY_EVENT_CAT1,
					barColor: win().barColor,
					isFullList: true,
				}});
			
			page.addField({ type: 'dropdown', name: 'Category 2', mandatory: false, dataKey:appProps.KEY_EVENT_CAT2,
				windowOptions: {
				    id: 'win_event_category',
				    url: 'filter_by_category.js',
					title:'Select a category',
					returnOnSelect:true,
					returnValueKey:appProps.KEY_EVENT_CAT2,
					barColor: win().barColor,
					isFullList: true,
				}});

			page.addSpacer(15);

			page.title = 'Step 3 of 4';
			page.leftButton = makePrevButton();
			page.rightButton = makeNextButton();
			break;
			
		case 4:
			page.setCancelButtonVisible(true);
			page.addHeading('Personal contact details');
			page.addField({ type: 'text', name: 'Name', mandatory: false, dataKey: appProps.KEY_EVENT_PERSON_NAME, });
			page.addField({ type: 'email', name: 'Email', mandatory: false, dataKey: appProps.KEY_EVENT_PERSON_EMAIL, });
			page.addField({ type: 'phone', name: 'Telephone', mandatory: false, dataKey: appProps.KEY_EVENT_PERSON_PHONE, });
			page.addSpacer(15);

			page.title = 'Step 4 of 4';
			page.leftButton = makePrevButton();
			page.rightButton = makeNextButton('submit', function(){
			
				if(page.validate())
				{
					page.saveTextFieldValues();
					showLoadingWindow();
				
					sendSuggestEvent(function(success){
						if(success === true)
						{
							page.saveTextFieldValues();
							currentStep++;
							showPage();
							hideLoadingWindow();
						}
						else
						{
							hideLoadingWindow();
							alert('Sorry, your event could not be submitted at this time.');
						}
						
						
					});
				}
			});
			break;
			
		case 5:
			page.setCancelButtonVisible(false);
			page.addHeading('Thank you for submitting details of your event');
			page.addLabel('We reserve the right to not publish all events submitted and priority is given to free or low cost events which are accessible to the general public. We also reserve the right to edit the copy submitted, purely to keep to a general standard, length and format.');
			page.addSpacer(15);
			
			if(!isIPad())
			{
				page.fullButton = makeNextButton('ok', function(){ win().close(); });
			}
			break;
			
		default:
			page.addHeading('Unknown page');
			page.addSpacer(10);
			page.leftButton = makePrevButton();
			break;
			

	}	
	page.render();
}

function updateScreen(traceMessageSource)
{
	Ti.API.trace('Suggest event called updateScreen (' + traceMessageSource + ') - currentStep is ' + currentStep); 
	page.loadTextFieldValues();
}

// Each device hooks different events due to Android vs iOS and iPhone nav group vs iPad splitview.
if(isIPad()) {
	$(win()).open(function() { updateScreen('iPad open'); });
} else if(isIPhone()) {
	$(win()).focus(function() { updateScreen('iPhone focus'); });
} else if(isAndroid()) {
	Titanium.Android.currentActivity.addEventListener("start", function(){ updateScreen('Android activity start'); });
}