Ti.include('../redux.js');

var filterHeading = null;
var isInMonthView = (win().isCalendarMonth ? true : false);
var filteredEventLocations = [];

function showCategorySubFilter(event)
{
	var newWin = new Window(mixinWindowOptions({
	    id: 'win_categories',
	    url: 'filter_by_category.js',
		title:'Categories',
		isSetAndReturn:true,
		barColor: win().barColor,
		selectedDate: getSelectedDate(),
	}));
	
	loadWindow(newWin);
}

win().layout = 'vertical';

// Get the primary filter text - e.g. "Awards" or "Melbourne City Council" - to display
// in the calendar header bar. The only exception is if this page has a separate
// category filter widget (i.e. win().showCategoryFilter == true) then don't display
// the primary filter bar
var categoryBarText = getPrimaryFilter();
if(win().showCategoryFilter) categoryBarText = null;

var calendarHeader = calendarHeaderView(categoryBarText, true, win().isCalendarMonth, isInMonthView);

// Because we may be swapping the header in and out if the user swaps between
// a "whole month" and "one day" view, put it in a container so that we can just
// add/remove from this container without having to mess with the whole layout

var calendarContainer = new View({ height: calendarHeader.height });
calendarContainer.add(calendarHeader);
win().add(calendarContainer);

var topSectionHeight = calendarContainer.height;

if(win().showCategoryFilter)
{
	filterHeading = makeCategoryFilter();
	win().add(filterHeading.filterView);
	topSectionHeight += filterHeading.filterView.height;
}

var tableView = makeFullscreenTable(topSectionHeight);
tableView.separatorColor = 'transparent';
if(isAndroid()) {
	tableView.backgroundSelectedColor = '#e0e0e0';
	tableView.footerView = Ti.UI.createView({height:70,width:getScreenWidth(),backgroundColor:'transparent', visible:false});
}
win().add(tableView);

refreshTableView();

var hasCriteriaChanged = false;
// When the window is focused, refresh the results
// Load stored values when the window receives focus
$(win()).open(function(){ 
	if(isIPad()) {
		updateScreen();
	}
});

// iPad will trigger this on first open of window. Stop doing that as it doubles load times.
$(win()).focus(function(){ 
	if(!isIPad()) {	
		updateScreen();
	}
});

function updateScreen(){

	if(hasCriteriaChanged)
	{
		refreshTableView();
		hasCriteriaChanged = false;
	}
};


// Add swipe detection to the window. Note that the swipe detection
// is tracked and duplicate events are prevented from firing for a second
// to prevent the event firing twice in a row (TI bug?)
var swipeDirection = 'none';
win().addEventListener('swipe', function(event){
	if(event.direction != swipeDirection)
	{
		swipeDirection = event.direction;
		if(event.direction === 'left')
		{
			setSelectedDate(nextDay(getSelectedDate()));
			refreshTableView()
		}
		else if(event.direction === 'right')
		{
			setSelectedDate(previousDay(getSelectedDate()));
			refreshTableView()
		}
		setTimeout(function() { swipeDirection = 'none'; }, 1000);
	}
});

Ti.App.addEventListener('filterCriteriaChanged', function(e){
	Ti.API.debug('filterCriteriaChanged fired');

	// Optional property "isWholeMonth" signifies a possible transition between
	// an "all month" and specific day view
	if(e.isWholeMonth != null)
	{
		if(isInMonthView != e.isWholeMonth)
		{
			Ti.API.debug('filterCriteriaChanged: new e.isWholeMonth=' + e.isWholeMonth);
			isInMonthView = e.isWholeMonth;
			var newCalendarHeader = calendarHeaderView(getPrimaryFilter(), true, win().isCalendarMonth, isInMonthView);
			calendarContainer.add(newCalendarHeader);
			calendarContainer.remove(calendarHeader);
			calendarHeader = newCalendarHeader;
		}
	}
	
	hasCriteriaChanged = true;
});

function refreshTableView() {	
	Ti.App.fireEvent('updateCalendarHeader');
	
	if(filterHeading != null) {
		filterHeading.update();
	}
	
	showLoading(tableView);

	// IV-75: Only show the date fields at the top if the user is looking at the whole month
	var fields = isInMonthView ? evtField.FieldsWithDates : evtField.NormalFields;
	loadRemoteEventsWithFilters(tableView, fields, false, isInMonthView);
}
